/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.core;

import java.util.LinkedList;
import java.util.List;

import javax.media.j3d.Transform3D;

public class RubikBlock {
	
	private RubikBlockID ID = null;
	private List<RubikBlockSide> sides = new LinkedList<RubikBlockSide>();
	private RubikCoordinates coords = null;
	
	public RubikBlock(RubikBlockID BLOCK_ID) {
		ID = BLOCK_ID;
		for(int i=0; i<6; i++) {
			sides.add(new RubikBlockSide());
		}
		sides.get(0).setDirection(RubikDirection.Z_P);
		sides.get(0).setColor(null);
		sides.get(1).setDirection(RubikDirection.Z_N);
		sides.get(1).setColor(null);
		sides.get(2).setDirection(RubikDirection.X_P);
		sides.get(2).setColor(null);
		sides.get(3).setDirection(RubikDirection.X_N);
		sides.get(3).setColor(null);
		sides.get(4).setDirection(RubikDirection.Y_P);
		sides.get(4).setColor(null);
		sides.get(5).setDirection(RubikDirection.Y_N);
		sides.get(5).setColor(null);
	}
	
	public RubikCoordinates getCoordinates() {
		return coords;
	}
	
	public Integer[] getCoordinates_Iarray() {
		return coords.getCoordinates_Iarray();
	}
	
	public RubikBlockID getID() {
		return ID;
	}
	
	public RubikBlockSide getSide(RubikDirection Rdirection) {
		for(RubikBlockSide curSide : sides) {
			if(curSide.getDirection() == Rdirection) {
				return curSide;
			}
		}
		System.err.println("getSide: Wrong argument");
		return null;
	}
	
	public void setCoordinates(Integer x, Integer y, Integer z) {
		if(coords == null) {
			coords = new RubikCoordinates(x, y, z);
		}
		else {
			coords.setCoordinates(x, y, z);
		}
	}
	
	public void setCoordinates(Integer[] COORDS) {
		if(coords == null) {
			coords = new RubikCoordinates(COORDS);
		}
		else {
			coords.setCoordinates(COORDS);
		}
	}
	
	public void setCoordinates(RubikCoordinates Rcoords) {
		coords = Rcoords;
	}
	
	public void setRotation(int x, int y, int z) {
		
		RubikColor temp;
		
		if(x == 1) {
			temp = getSide(RubikDirection.Z_P).getColor();
			getSide(RubikDirection.Z_P).setColor(
					getSide(RubikDirection.Y_P).getColor());
			getSide(RubikDirection.Y_P).setColor(
					getSide(RubikDirection.Z_N).getColor());
			getSide(RubikDirection.Z_N).setColor(
					getSide(RubikDirection.Y_N).getColor());
			getSide(RubikDirection.Y_N).setColor(temp);
		}
		else if(x == -1) {
			temp = getSide(RubikDirection.Z_P).getColor();
			getSide(RubikDirection.Z_P).setColor(
					getSide(RubikDirection.Y_N).getColor());
			getSide(RubikDirection.Y_N).setColor(
					getSide(RubikDirection.Z_N).getColor());
			getSide(RubikDirection.Z_N).setColor(
					getSide(RubikDirection.Y_P).getColor());
			getSide(RubikDirection.Y_P).setColor(temp);
		}
		else if(y == 1) {
			temp = getSide(RubikDirection.Z_P).getColor();
			getSide(RubikDirection.Z_P).setColor(
					getSide(RubikDirection.X_N).getColor());
			getSide(RubikDirection.X_N).setColor(
					getSide(RubikDirection.Z_N).getColor());
			getSide(RubikDirection.Z_N).setColor(
					getSide(RubikDirection.X_P).getColor());
			getSide(RubikDirection.X_P).setColor(temp);
		}
		else if(y == -1) {
			temp = getSide(RubikDirection.Z_P).getColor();
			getSide(RubikDirection.Z_P).setColor(
					getSide(RubikDirection.X_P).getColor());
			getSide(RubikDirection.X_P).setColor(
					getSide(RubikDirection.Z_N).getColor());
			getSide(RubikDirection.Z_N).setColor(
					getSide(RubikDirection.X_N).getColor());
			getSide(RubikDirection.X_N).setColor(temp);
		}
		else if(z == 1) {
			temp = getSide(RubikDirection.Y_P).getColor();
			getSide(RubikDirection.Y_P).setColor(
					getSide(RubikDirection.X_P).getColor());
			getSide(RubikDirection.X_P).setColor(
					getSide(RubikDirection.Y_N).getColor());
			getSide(RubikDirection.Y_N).setColor(
					getSide(RubikDirection.X_N).getColor());
			getSide(RubikDirection.X_N).setColor(temp);
		}
		else if(z == -1) {
			temp = getSide(RubikDirection.Y_P).getColor();
			getSide(RubikDirection.Y_P).setColor(
					getSide(RubikDirection.X_N).getColor());
			getSide(RubikDirection.X_N).setColor(
					getSide(RubikDirection.Y_N).getColor());
			getSide(RubikDirection.Y_N).setColor(
					getSide(RubikDirection.X_P).getColor());
			getSide(RubikDirection.X_P).setColor(temp);
		}
		
	}
}
