/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.core;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Random;
import java.util.Stack;

import rubik.core.rubikgroup.RubikDisjointCycleDecomposition;

public class RubikCube {

	//TODO: make this private and define a get fn
	public List<RubikBlock> blocks = new LinkedList<RubikBlock>();
	public Map<Integer[], Integer> coordinateMap = new HashMap<Integer[], Integer>(26);
	
	static Random rand = null;
	
	RubikRotationList solution = null;

	public RubikCube() {

		createBlocks();
		assignBlockSides();
		rand = new Random();

	}

	private void assignBlockSides() {
		//set coordinates for all blocks
		blocks.get(0).setCoordinates(RubikCoordinates.c0_0_2);
		blocks.get(1).setCoordinates(RubikCoordinates.c2_0_2);
		blocks.get(2).setCoordinates(RubikCoordinates.c2_2_2);
		blocks.get(3).setCoordinates(RubikCoordinates.c0_2_2);
		blocks.get(4).setCoordinates(RubikCoordinates.c0_0_0);
		blocks.get(5).setCoordinates(RubikCoordinates.c2_0_0);
		blocks.get(6).setCoordinates(RubikCoordinates.c2_2_0);
		blocks.get(7).setCoordinates(RubikCoordinates.c0_2_0);
		blocks.get(8).setCoordinates(RubikCoordinates.c1_1_2);
		blocks.get(9).setCoordinates(RubikCoordinates.c1_0_1);
		blocks.get(10).setCoordinates(RubikCoordinates.c2_1_1);
		blocks.get(11).setCoordinates(RubikCoordinates.c1_2_1);
		blocks.get(12).setCoordinates(RubikCoordinates.c0_1_1);
		blocks.get(13).setCoordinates(RubikCoordinates.c1_1_0);
		blocks.get(14).setCoordinates(RubikCoordinates.c1_0_2);
		blocks.get(15).setCoordinates(RubikCoordinates.c2_1_2);
		blocks.get(16).setCoordinates(RubikCoordinates.c1_2_2);
		blocks.get(17).setCoordinates(RubikCoordinates.c0_1_2);
		blocks.get(18).setCoordinates(RubikCoordinates.c0_0_1);
		blocks.get(19).setCoordinates(RubikCoordinates.c2_0_1);
		blocks.get(20).setCoordinates(RubikCoordinates.c2_2_1);
		blocks.get(21).setCoordinates(RubikCoordinates.c0_2_1);
		blocks.get(22).setCoordinates(RubikCoordinates.c1_0_0);
		blocks.get(23).setCoordinates(RubikCoordinates.c2_1_0);
		blocks.get(24).setCoordinates(RubikCoordinates.c1_2_0);
		blocks.get(25).setCoordinates(RubikCoordinates.c0_1_0);

		//assign colors to cube corner blocks
		blocks.get(0).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);
		blocks.get(0).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);
		blocks.get(0).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);

		blocks.get(1).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);
		blocks.get(1).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);
		blocks.get(1).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);

		blocks.get(2).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);
		blocks.get(2).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);
		blocks.get(2).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);

		blocks.get(3).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);
		blocks.get(3).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);
		blocks.get(3).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);

		blocks.get(4).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
		blocks.get(4).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);
		blocks.get(4).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);

		blocks.get(5).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
		blocks.get(5).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);
		blocks.get(5).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);

		blocks.get(6).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
		blocks.get(6).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);
		blocks.get(6).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);

		blocks.get(7).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
		blocks.get(7).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);
		blocks.get(7).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);

		//assign colors to center blocks
		blocks.get(8).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);

		blocks.get(9).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);

		blocks.get(10).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);

		blocks.get(11).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);

		blocks.get(12).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);

		blocks.get(13).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);

		//assign colors to side corner blocks
		blocks.get(14).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);
		blocks.get(14).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);

		blocks.get(15).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);
		blocks.get(15).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);

		blocks.get(16).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);
		blocks.get(16).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);

		blocks.get(17).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);
		blocks.get(17).getSide(RubikDirection.Z_P).setColor(RubikColor.YELLOW);

		blocks.get(18).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);
		blocks.get(18).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);

		blocks.get(19).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);
		blocks.get(19).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);

		blocks.get(20).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);
		blocks.get(20).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);

		blocks.get(21).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);
		blocks.get(21).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);

		blocks.get(22).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
		blocks.get(22).getSide(RubikDirection.Y_N).setColor(RubikColor.RED);

		blocks.get(23).getSide(RubikDirection.X_P).setColor(RubikColor.GREEN);
		blocks.get(23).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);

		blocks.get(24).getSide(RubikDirection.Y_P).setColor(RubikColor.ORANGE);
		blocks.get(24).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);

		blocks.get(25).getSide(RubikDirection.X_N).setColor(RubikColor.BLUE);
		blocks.get(25).getSide(RubikDirection.Z_N).setColor(RubikColor.WHITE);
	}

	private void createBlocks() {
		if(!blocks.isEmpty()) {
			blocks.clear();
		}

		//blocks 0-7 cube corner blocks
		for(int i=0; i<8; i++) {
			blocks.add(new RubikBlock(RubikBlockID.CORNER_BLOCK));
		}

		//blocks 8-13 side center blocks
		for(int i=0; i<6; i++) {
			blocks.add(new RubikBlock(RubikBlockID.SIDE_CENTER));
		}

		//blocks 14-25 side corner blocks
		for(int i=0; i<12; i++ ) {
			blocks.add(new RubikBlock(RubikBlockID.SIDE_CORNER));
		}

		coordinateMap.put(RubikCoordinates.c0_0_2, 0);
		coordinateMap.put(RubikCoordinates.c2_0_2, 1);
		coordinateMap.put(RubikCoordinates.c2_2_2, 2);
		coordinateMap.put(RubikCoordinates.c0_2_2, 3);
		coordinateMap.put(RubikCoordinates.c0_0_0, 4);
		coordinateMap.put(RubikCoordinates.c2_0_0, 5);
		coordinateMap.put(RubikCoordinates.c2_2_0, 6);
		coordinateMap.put(RubikCoordinates.c0_2_0, 7);
		coordinateMap.put(RubikCoordinates.c1_1_2, 8);
		coordinateMap.put(RubikCoordinates.c1_0_1, 9);
		coordinateMap.put(RubikCoordinates.c2_1_1, 10);
		coordinateMap.put(RubikCoordinates.c1_2_1, 11);
		coordinateMap.put(RubikCoordinates.c0_1_1, 12);
		coordinateMap.put(RubikCoordinates.c1_1_0, 13);
		coordinateMap.put(RubikCoordinates.c1_0_2, 14);
		coordinateMap.put(RubikCoordinates.c2_1_2, 15);
		coordinateMap.put(RubikCoordinates.c1_2_2, 16);
		coordinateMap.put(RubikCoordinates.c0_1_2, 17);
		coordinateMap.put(RubikCoordinates.c0_0_1, 18);
		coordinateMap.put(RubikCoordinates.c2_0_1, 19);
		coordinateMap.put(RubikCoordinates.c2_2_1, 20);
		coordinateMap.put(RubikCoordinates.c0_2_1, 21);
		coordinateMap.put(RubikCoordinates.c1_0_0, 22);
		coordinateMap.put(RubikCoordinates.c2_1_0, 23);
		coordinateMap.put(RubikCoordinates.c1_2_0, 24);
		coordinateMap.put(RubikCoordinates.c0_1_0, 25);

	}
	
	public void randomize(int depth) {
		int count = 0;
		List<RubikDisjointCycleDecomposition> states = 
			new LinkedList<RubikDisjointCycleDecomposition>();
		RubikDisjointCycleDecomposition prev = new RubikDisjointCycleDecomposition(true);
		states.add(prev);
		RubikDisjointCycleDecomposition cur, next;
		RubikRotationList temp = new RubikRotationList();
		RubikRotationList commands = new RubikRotationList();
		RubikRotationList revCommands = new RubikRotationList();
		int command;
		boolean done;
		
		command = getRandomCommand();
		temp.getList().clear();
		temp.add(command);
		cur = (RubikDisjointCycleDecomposition)prev.clone();
		cur.applyCommands(temp);
		states.add(cur);
		
		commands.add(command);
		
		while(count < depth) {
			do {
				command = getRandomCommand();
				temp.getList().clear();
				temp.add(command);
				next = (RubikDisjointCycleDecomposition)cur.clone();
				next.applyCommands(temp);
			}while(next.getBlockRDC().differenceWith(prev.getBlockRDC()) < 11);
			done = true;
			for(RubikDisjointCycleDecomposition p : states) {
				if(next.similarityWith(p) > 13 && p!=states.get(states.size()-1)) {
					done = false;
					break;
				}
			}
			if(done) {
				commands.add(command);
				prev = cur;
				cur = next;
				states.add(next);
				count++;
			}
			if(next.similarityWith(states.get(0)) < 3 && count == depth) {
				count--;
			}
		}
		applyCommands(commands);
		for(Integer i: commands.getList()) {
			System.out.print(RubikRotationList.getText(i) + " ");
		}
		System.out.println(commands.getList().size());
		for(int i=commands.getList().size()-1; i>=0; i--) {
			int cmd = commands.getList().get(i);
			if(cmd%2 == 0) {
				System.out.print(RubikRotationList.getText(cmd+1) + " ");
				revCommands.add(cmd+1);
			}
			else {
				System.out.print(RubikRotationList.getText(cmd-1) + " ");
				revCommands.add(cmd-1);
			}
			
		}
		System.out.println(commands.getList().size());
		revCommands.reduce();
		for(Integer i: revCommands.getList()) {
			System.out.print(RubikRotationList.getText(i) + " ");
		}
		System.out.println(revCommands.getList().size());
//		applyCommands(revCommands);
	}
	
	int getRandomCommand() {
		int random;
		int command;
		random = rand.nextInt(65536);
		final float interval = (float)65536/12;

		if(random < (int)interval) {
			command = RubikRotationList.e3;
		}
		else if(random >= (int)2*interval &&
				random < (int)3*interval) {
			command = RubikRotationList.e5;
		}
		else if(random >= (int)3*interval &&
				random < (int)4*interval) {
			command = RubikRotationList.c3;
		}
		else if(random >= (int)4*interval &&
				random < (int)5*interval/2) {
			command = RubikRotationList.c5;
		}
		else if(random >= (int)5*interval &&
				random < (int)6*interval) {
			command = RubikRotationList.r3;
		}
		else if(random >= (int)6*interval &&
				random < (int)7*interval){
			command = RubikRotationList.r5;
		}
		else if(random >= (int)7*interval &&
				random < (int)8*interval) {
			command = RubikRotationList.v3;
		}
		else if(random >= (int)8*interval &&
				random < (int)9*interval) {
			command = RubikRotationList.v5;
		}
		else if(random >= (int)9*interval &&
				random < (int)10*interval) {
			command = RubikRotationList.t3;
		}
		else if(random >= (int)10*interval &&
				random < (int)11*interval) {
			command = RubikRotationList.t5;
		}
		else if(random >= (int)11*interval &&
				random < (int)12*interval) {
			command = RubikRotationList.b3;
		}
		else{
			command = RubikRotationList.b5;
		}

		return command;
	}
	
	//TODO: Make this efficient !!!!
	public void applyCommands(RubikRotationList list) {
		Map<Integer[], Integer> newCoordinateMap = new HashMap<Integer[], Integer>();
		List<Integer[]> coordstobeRemoved = new LinkedList<Integer[]>();
		Integer[] tempCoords = null;
		Integer[] newCoords = null;
		Integer tempVal = null;
		Stack<Integer> rotationList = list.getList();
		ListIterator<Integer> iter = rotationList.listIterator();
		int next;
		while(iter.hasNext()) {
			next = iter.next();
			if(next == RubikRotationList.e3 ||
					next == RubikRotationList.e5 ||
					next == RubikRotationList.d3 ||
					next == RubikRotationList.d5 ||
					next == RubikRotationList.c3 ||
					next == RubikRotationList.c5) {
				for(RubikBlock block: blocks) {
					if( (block.getCoordinates().getz() == 2 && (next == RubikRotationList.e3 || next == RubikRotationList.e5)) ||
							(block.getCoordinates().getz() == 1 && (next == RubikRotationList.d3 || next == RubikRotationList.d5)) ||
							(block.getCoordinates().getz() == 0 && (next == RubikRotationList.c3 || next == RubikRotationList.c5)) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = coordinateMap.get(tempCoords);

						if(next == RubikRotationList.e3 || next == RubikRotationList.d3 || next == RubikRotationList.c3) {
							newCoords = RubikRotationMaps.getZPMap().get(tempCoords);

							blocks.get(tempVal).setRotation(0, 0, 1);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getZNMap().get(tempCoords);

							blocks.get(tempVal).setRotation(0, 0, -1);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
					}
				}
			}
			else if(next == RubikRotationList.r3 ||
					next == RubikRotationList.r5 ||
					next == RubikRotationList.f3 ||
					next == RubikRotationList.f5 ||
					next == RubikRotationList.v3 ||
					next == RubikRotationList.v5) {
				for(RubikBlock block: blocks) {
					if( (block.getCoordinates().getx() == 2 && (next == RubikRotationList.r3 || next == RubikRotationList.r5)) ||
							(block.getCoordinates().getx() == 1 && (next == RubikRotationList.f3 || next == RubikRotationList.f5)) ||
							(block.getCoordinates().getx() == 0 && (next == RubikRotationList.v3 || next == RubikRotationList.v5)) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = coordinateMap.get(tempCoords);

						if(next == RubikRotationList.r3 || next == RubikRotationList.f3 || next == RubikRotationList.v3) {
							newCoords = RubikRotationMaps.getXPMap().get(tempCoords);

							blocks.get(tempVal).setRotation(1, 0, 0);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getXNMap().get(tempCoords);

							blocks.get(tempVal).setRotation(-1, 0, 0);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
					}
				}
			}
			else if(next == RubikRotationList.t3 ||
					next == RubikRotationList.t5 ||
					next == RubikRotationList.g3 ||
					next == RubikRotationList.g5 ||
					next == RubikRotationList.b3 ||
					next == RubikRotationList.b5) {
				for(RubikBlock block: blocks) {
					if( (block.getCoordinates().gety() == 2 && (next == RubikRotationList.t3 || next == RubikRotationList.t5)) ||
							(block.getCoordinates().gety() == 1 && (next == RubikRotationList.g3 || next == RubikRotationList.g5)) ||
							(block.getCoordinates().gety() == 0 && (next == RubikRotationList.b3 || next == RubikRotationList.b5)) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = coordinateMap.get(tempCoords);

						if(next == RubikRotationList.t3 || next == RubikRotationList.g3 || next == RubikRotationList.b3) {
							newCoords = RubikRotationMaps.getYPMap().get(tempCoords);

							blocks.get(tempVal).setRotation(0, 1, 0);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getYNMap().get(tempCoords);

							blocks.get(tempVal).setRotation(0, -1, 0);
							blocks.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
					}
				}
			}


			for(Integer[] coords: coordstobeRemoved) {
				coordinateMap.remove(coords);
			}
			coordinateMap.putAll(newCoordinateMap);

		}

	}


}
