/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.core;


import java.util.Map;
import java.util.Stack;

public class RubikRotationList {
	
	public static final int e3 = 0;
	public static final int e5 = 1;
	public static final int d3 = 2;
	public static final int d5 = 3;
	public static final int c3 = 4;
	public static final int c5 = 5;
	public static final int r3 = 6;
	public static final int r5 = 7;
	public static final int f3 = 8;
	public static final int f5 = 9;
	public static final int v3 = 10;
	public static final int v5 = 11;
	public static final int t3 = 12;
	public static final int t5 = 13;
	public static final int g3 = 14;
	public static final int g5 = 15;
	public static final int b3 = 16;
	public static final int b5 = 17;
	
	static String[] commandsText = {"E3", "E5", "D3", "D5", "C3", "C5",
									"R3", "R5", "F3", "F5", "V3", "V5",
									"T3", "T5", "G3", "G5", "B3", "B5"};
	
	Stack<Integer> rotationList = null;
	
	public RubikRotationList() {
		rotationList = new Stack<Integer>();
	}
	
	public int add(Integer i) {
		if(i >= 0 && i <= 18) {
			rotationList.push(i);
			return 0;
		}
		else {
			return -1;
		}
	}
	
	public Stack<Integer> getList() {
		return rotationList;
	}
	
	public int pop() {
		if(!rotationList.isEmpty()) {
			return rotationList.pop().intValue();
		}
		else {
			return -1;
		}
		
	}
	
	public void reduce() {
		Map<Integer, Integer> map = null;
		for(int i=0; i<rotationList.size(); i++) {
			if(i<rotationList.size()-2) {
				if((rotationList.get(i) == e3 && rotationList.get(i+1) == c3) ||
						(rotationList.get(i) == c3 && rotationList.get(i+1) == e3)) {
					map = RubikListReductionMaps.gete3c3Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(i);
					rotationList.insertElementAt(d5, i);
				}
				else if((rotationList.get(i) == e5 && rotationList.get(i+1) == c5) ||
						(rotationList.get(i) == c5 && rotationList.get(i+1) == e5)) {
					map = RubikListReductionMaps.gete5c5Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(i);
					rotationList.insertElementAt(d3, i);
				}
				else if((rotationList.get(i) == r3 && rotationList.get(i+1) == v3) ||
						(rotationList.get(i) == v3 && rotationList.get(i+1) == r3)) {
					map = RubikListReductionMaps.getr3v3Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(i);
					rotationList.insertElementAt(f5, i);
				}
				else if((rotationList.get(i) == r5 && rotationList.get(i+1) == v5) ||
						(rotationList.get(i) == v5 && rotationList.get(i+1) == r5)) {
					map = RubikListReductionMaps.getr5v5Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(i);
					rotationList.insertElementAt(f3, i);
				}
				else if((rotationList.get(i) == t3 && rotationList.get(i+1) == b3) ||
						(rotationList.get(i) == b3 && rotationList.get(i+1) == t3)) {
					map = RubikListReductionMaps.gett3b3Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(i);
					rotationList.insertElementAt(g5, i);
				}
				else if((rotationList.get(i) == t5 && rotationList.get(i+1) == b5) ||
						(rotationList.get(i) == b5 && rotationList.get(i+1) == t5)) {
					map = RubikListReductionMaps.gett5b5Map();
					for(int j=i+2; j<rotationList.size(); j++) {
						rotationList.set(j, map.get(rotationList.get(j)));
					}
					rotationList.remove(i);
					rotationList.remove(1);
					rotationList.insertElementAt(g3, i);
				}
			}
		}
	}
	
	public boolean isEmpty() {
		return rotationList.isEmpty();
	}
	
	public static String getText(int command) {
		if(command >= 0 && command <= 17) {
			return commandsText[command];
		}
		else {
			return "Wrong Command!";
		}
	}
}
