/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.core.rubikgroup;

import rubik.core.RubikRotationList;

public class RubikDisjointCycleDecomposition {
	RubikDisjointCycle blocks = null;
	RubikDisjointCycle[] sides = null;
	
	
	public RubikDisjointCycleDecomposition(boolean createEmptyRDCs) {
		sides = new RubikDisjointCycle[26];
		if(createEmptyRDCs) {
			blocks = new RubikDisjointCycle(RubikDisjointCycle.BLOCKS);

			for(int i=0; i<26; i++) {
				sides[i] = new RubikDisjointCycle(RubikDisjointCycle.SIDES);
			}
		}
	}
	
	public int similarityWith(RubikDisjointCycleDecomposition rdcd) {
		int ret = 0;
		boolean mat1[][] = rdcd.getBlockRDC().getTransformationMatrix();
		boolean mat2[][] = blocks.getTransformationMatrix();
		for(int i=0; i<RubikDisjointCycle.blockArraySize; i++) {
			for(int j=0; j<RubikDisjointCycle.blockArraySize; j++) {
				if(mat1[i][j] == RubikDisjointCycle.c1 || mat2[i][j] == RubikDisjointCycle.c1) {
					if(mat1[i][j] == RubikDisjointCycle.c1 && mat2[i][j] == RubikDisjointCycle.c1) {
						if(sides[i].equals(rdcd.getSideRDC(i))) {
							ret++;
						}
					}
					break;
				}
			}
		}
		return ret;
	}
	
	public void applyCommands(RubikRotationList l1) {
		RubikRotationList curCmd = new RubikRotationList();
		for(int cmd: l1.getList()) {
			curCmd.getList().clear();
			curCmd.add(cmd);
			blocks.applyCommands(curCmd);
			for(int i=0; i<26; i++) {
				if(cmd == RubikRotationList.e3 ||
						cmd == RubikRotationList.e5) {
					sides[getBlockatPos(0)].applyCommands(curCmd);
					sides[getBlockatPos(1)].applyCommands(curCmd);
					sides[getBlockatPos(2)].applyCommands(curCmd);
					sides[getBlockatPos(3)].applyCommands(curCmd);
					sides[getBlockatPos(14)].applyCommands(curCmd);
					sides[getBlockatPos(15)].applyCommands(curCmd);
					sides[getBlockatPos(16)].applyCommands(curCmd);
					sides[getBlockatPos(17)].applyCommands(curCmd);
					sides[getBlockatPos(8)].applyCommands(curCmd);
					
				}
				else if(cmd == RubikRotationList.d3 ||
						cmd == RubikRotationList.d5) {
					sides[getBlockatPos(9)].applyCommands(curCmd);
					sides[getBlockatPos(10)].applyCommands(curCmd);
					sides[getBlockatPos(11)].applyCommands(curCmd);
					sides[getBlockatPos(12)].applyCommands(curCmd);
					sides[getBlockatPos(18)].applyCommands(curCmd);
					sides[getBlockatPos(19)].applyCommands(curCmd);
					sides[getBlockatPos(20)].applyCommands(curCmd);
					sides[getBlockatPos(21)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.c3 ||
						cmd == RubikRotationList.c5) {
					sides[getBlockatPos(4)].applyCommands(curCmd);
					sides[getBlockatPos(5)].applyCommands(curCmd);
					sides[getBlockatPos(6)].applyCommands(curCmd);
					sides[getBlockatPos(7)].applyCommands(curCmd);
					sides[getBlockatPos(22)].applyCommands(curCmd);
					sides[getBlockatPos(23)].applyCommands(curCmd);
					sides[getBlockatPos(24)].applyCommands(curCmd);
					sides[getBlockatPos(25)].applyCommands(curCmd);
					sides[getBlockatPos(13)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.r3 ||
						cmd == RubikRotationList.r5) {
					sides[getBlockatPos(6)].applyCommands(curCmd);
					sides[getBlockatPos(1)].applyCommands(curCmd);
					sides[getBlockatPos(2)].applyCommands(curCmd);
					sides[getBlockatPos(5)].applyCommands(curCmd);
					sides[getBlockatPos(20)].applyCommands(curCmd);
					sides[getBlockatPos(15)].applyCommands(curCmd);
					sides[getBlockatPos(19)].applyCommands(curCmd);
					sides[getBlockatPos(23)].applyCommands(curCmd);
					sides[getBlockatPos(10)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.f3 ||
						cmd == RubikRotationList.f5) {
					sides[getBlockatPos(9)].applyCommands(curCmd);
					sides[getBlockatPos(14)].applyCommands(curCmd);
					sides[getBlockatPos(8)].applyCommands(curCmd);
					sides[getBlockatPos(16)].applyCommands(curCmd);
					sides[getBlockatPos(11)].applyCommands(curCmd);
					sides[getBlockatPos(24)].applyCommands(curCmd);
					sides[getBlockatPos(13)].applyCommands(curCmd);
					sides[getBlockatPos(22)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.v3 ||
						cmd == RubikRotationList.v5) {
					sides[getBlockatPos(0)].applyCommands(curCmd);
					sides[getBlockatPos(18)].applyCommands(curCmd);
					sides[getBlockatPos(17)].applyCommands(curCmd);
					sides[getBlockatPos(3)].applyCommands(curCmd);
					sides[getBlockatPos(21)].applyCommands(curCmd);
					sides[getBlockatPos(7)].applyCommands(curCmd);
					sides[getBlockatPos(25)].applyCommands(curCmd);
					sides[getBlockatPos(4)].applyCommands(curCmd);
					sides[getBlockatPos(12)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.t3 ||
						cmd == RubikRotationList.t5) {
					sides[getBlockatPos(21)].applyCommands(curCmd);
					sides[getBlockatPos(3)].applyCommands(curCmd);
					sides[getBlockatPos(16)].applyCommands(curCmd);
					sides[getBlockatPos(2)].applyCommands(curCmd);
					sides[getBlockatPos(20)].applyCommands(curCmd);
					sides[getBlockatPos(7)].applyCommands(curCmd);
					sides[getBlockatPos(24)].applyCommands(curCmd);
					sides[getBlockatPos(6)].applyCommands(curCmd);
					sides[getBlockatPos(11)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.g3 ||
						cmd == RubikRotationList.g5) {
					sides[getBlockatPos(12)].applyCommands(curCmd);
					sides[getBlockatPos(17)].applyCommands(curCmd);
					sides[getBlockatPos(8)].applyCommands(curCmd);
					sides[getBlockatPos(15)].applyCommands(curCmd);
					sides[getBlockatPos(10)].applyCommands(curCmd);
					sides[getBlockatPos(25)].applyCommands(curCmd);
					sides[getBlockatPos(13)].applyCommands(curCmd);
					sides[getBlockatPos(23)].applyCommands(curCmd);
				}
				else if(cmd == RubikRotationList.b3 ||
						cmd == RubikRotationList.b5) {
					sides[getBlockatPos(0)].applyCommands(curCmd);
					sides[getBlockatPos(1)].applyCommands(curCmd);
					sides[getBlockatPos(18)].applyCommands(curCmd);
					sides[getBlockatPos(19)].applyCommands(curCmd);
					sides[getBlockatPos(14)].applyCommands(curCmd);
					sides[getBlockatPos(4)].applyCommands(curCmd);
					sides[getBlockatPos(22)].applyCommands(curCmd);
					sides[getBlockatPos(5)].applyCommands(curCmd);
					sides[getBlockatPos(9)].applyCommands(curCmd);
				}
			}
		}
	}
	
	int getBlockatPos(int j) {
		for(int i=0; i<RubikDisjointCycle.blockArraySize; i++) {
			if(blocks.transformationMatrix[i][j] == RubikDisjointCycle.c1) {
				return i;
			}
		}
		return -1;
	}
	
	@Override
	public boolean equals(Object rdcd) {
		if(rdcd instanceof RubikDisjointCycleDecomposition) {
			RubikDisjointCycleDecomposition c = 
				(RubikDisjointCycleDecomposition) rdcd;
			if(!c.getBlockRDC().equals(blocks)) {
				return false;
			}
			for(int i=0; i<26; i++) {
				if(!c.getSideRDC(i).equals(sides[i])) {
					return false;
				}
			}
			return true;
		}
		else {
			return false;
		}
	}
	
	@Override
	public Object clone() {
		RubikDisjointCycleDecomposition ret = 
			new RubikDisjointCycleDecomposition(false);
		ret.blocks = (RubikDisjointCycle) blocks.clone();
		for(int i=0; i<26; i++) {
			ret.sides[i] = (RubikDisjointCycle) sides[i].clone(); 
		}
		return ret;
	}
	
	public RubikDisjointCycle getBlockRDC() {
		return blocks;
	}
	
	public RubikDisjointCycle getSideRDC(int num) {
		if(num < 0 || num >= 26) {
			return null;
		}
		else return sides[num];
	}
}
