/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.ui;

import java.awt.BorderLayout;
import java.awt.GraphicsConfigTemplate;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.GraphicsConfigTemplate3D;
import javax.media.j3d.Group;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.ImageComponent2D;
import javax.media.j3d.Locale;
import javax.media.j3d.PhysicalBody;
import javax.media.j3d.PhysicalEnvironment;
import javax.media.j3d.PointLight;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.View;
import javax.media.j3d.ViewPlatform;
import javax.media.j3d.VirtualUniverse;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.ToolTipManager;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.vecmath.Color3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3d;

import rubik.ui.settings.RubikSliderMenuElement;

public class RubikApplication extends JPanel implements ActionListener, ChangeListener {


	private static final long serialVersionUID = 1L;
	//	SimpleUniverse univ = null;
	BranchGroup root = new BranchGroup();
	RubikCube3D cube3D = null;
	boolean axisPresent = false;

	TransformGroup[] tgArray = null;
	Transform3D t3d_axis_true = null;
	Transform3D t3d_axis_false = null;

	JMenu fileMenu = null;
	JMenuItem showAxis = null;
	JMenuItem hideAxis = null;

	RubikSliderMenuElement slider = null;

	private Canvas3D offScreenCanvas3D = null;

	/**
	 * the width of the offscreen Canvas3D
	 */
	private static final int offScreenWidth = 400;

	/**
	 * the height of the offscreen Canvas3D
	 */
	private static final int offScreenHeight = 400;

	/**
	 * the image that is attached to the offscreen Canvas3D and contains the
	 * results of screen captures
	 */
	private ImageComponent2D imageComponent = null;

	public RubikApplication() {

		setLayout(new BorderLayout());

		VirtualUniverse universe = new VirtualUniverse();

		Locale locale = new Locale(universe);

		root.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		root.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);

		//		setLayout(new BorderLayout());
		//		GraphicsConfiguration config =
		//			SimpleUniverse.getPreferredConfiguration();
		//		Canvas3D canvas = new Canvas3D(config);
		//		add("Center", canvas);

		cube3D = new RubikCube3D(root);
		cube3D.setAxis(false);

		tgArray = new TransformGroup[2];

		t3d_axis_true = new Transform3D();
		t3d_axis_true.setScale(1);
		t3d_axis_true.setTranslation(new Vector3d(0.4, 0.0, -0.5));
		t3d_axis_true.invert();

		t3d_axis_false = new Transform3D();
		t3d_axis_false.setScale(1);
		t3d_axis_false.setTranslation(new Vector3d(0.0, 0.0, 0.0));
		t3d_axis_false.invert();

		showAxis = createMenuItem("R3", "Show Axis", this);
		hideAxis = createMenuItem("R3", "Hide Axis", this);

		//		Transform3D viewTransform = new Transform3D();
		//		viewTransform.setTranslation(new Vector3d(0, 0, 2.6));

		BoundingSphere bounds = new BoundingSphere();

		Color3f ambientColor = new Color3f(0.1f, 0.1f, 0.1f);
		AmbientLight ambientLightNode = new AmbientLight(ambientColor);
		ambientLightNode.setInfluencingBounds(bounds);
		root.addChild(ambientLightNode);

		Color3f light1Color = new Color3f(0.2f, 0.2f, 0.2f);

		PointLight pLight1 = new PointLight();
		pLight1.setColor(light1Color);
		pLight1.setAttenuation(1.2f, 0.1f, 0f);
		pLight1.setPosition(new Point3f(-1.5f, -1.5f, 1.5f));
		pLight1.setInfluencingBounds(bounds);

		PointLight pLight2 = new PointLight();
		pLight2.setColor(light1Color);
		pLight2.setAttenuation(1.2f, 0.1f, 0f);
		pLight2.setPosition(new Point3f(-1.5f, 1.5f, 1.5f));
		pLight2.setInfluencingBounds(bounds);

		PointLight pLight3 = new PointLight();
		pLight3.setColor(light1Color);
		pLight3.setAttenuation(1.2f, 0.1f, 0f);
		pLight3.setPosition(new Point3f(1.5f, -1.5f, 1.5f));
		pLight3.setInfluencingBounds(bounds);

		PointLight pLight4 = new PointLight();
		pLight4.setColor(light1Color);
		pLight4.setAttenuation(1.2f, 0.1f, 0f);
		pLight4.setPosition(new Point3f(1.5f, 1.5f, 1.5f));
		pLight4.setInfluencingBounds(bounds);

		root.addChild(pLight1);
		root.addChild(pLight2);
		root.addChild(pLight3);
		root.addChild(pLight4);

		root.compile();

		//		univ = new SimpleUniverse(canvas);
		//		univ.getViewingPlatform().setNominalViewingTransform();
		//		univ.getViewingPlatform().getViewPlatformTransform().setTransform(viewTransform);
		//		univ.addBranchGraph(root);

		ViewPlatform vp = createViewPlatform();


		BranchGroup viewBranchGroup = createViewBranchGroup(
				getViewTransformGroupArray(), vp);

		locale.addBranchGraph(root);
		locale.addBranchGraph(viewBranchGroup);

		createView(vp);

		System.gc();

	}

	protected View createView(ViewPlatform vp) {
		View view = new View();

		PhysicalBody pb = new PhysicalBody();
		PhysicalEnvironment pe = new PhysicalEnvironment();

		view.setPhysicalEnvironment(pe);
		view.setPhysicalBody(pb);

		if (vp != null)
			view.attachViewPlatform(vp);

		//		view.setBackClipDistance(3d);
		//		view.setFrontClipDistance(0d);

		// create the visible canvas
		Canvas3D c3d = createCanvas3D(false);
		view.addCanvas3D(c3d);

		// create the off screen canvas
		view.addCanvas3D(createOffscreenCanvas3D());

		// add the visible canvas to a component
		addCanvas3D(c3d);

		return view;
	}

	protected void addCanvas3D(Canvas3D c3d) {
		add(c3d, BorderLayout.CENTER);
	}

	protected Canvas3D createCanvas3D(boolean offscreen) {
		GraphicsConfigTemplate3D gc3D = new GraphicsConfigTemplate3D();
		gc3D.setSceneAntialiasing(GraphicsConfigTemplate.PREFERRED);
		GraphicsDevice gd[] = GraphicsEnvironment.getLocalGraphicsEnvironment()
		.getScreenDevices();

		Canvas3D c3d = new Canvas3D(gd[0].getBestConfiguration(gc3D), offscreen);
		c3d.setSize(500, 500);

		return c3d;
	}

	protected Canvas3D createOffscreenCanvas3D() {
		offScreenCanvas3D = createCanvas3D(true);
		offScreenCanvas3D.getScreen3D()
		.setSize(offScreenWidth, offScreenHeight);
		offScreenCanvas3D.getScreen3D().setPhysicalScreenHeight(
				0.0254 / 90 * offScreenHeight);
		offScreenCanvas3D.getScreen3D().setPhysicalScreenWidth(
				0.0254 / 90 * offScreenWidth);

		RenderedImage renderedImage = new BufferedImage(offScreenWidth,
				offScreenHeight, BufferedImage.TYPE_3BYTE_BGR);
		imageComponent = new ImageComponent2D(ImageComponent.FORMAT_RGB8,
				renderedImage);
		imageComponent.setCapability(ImageComponent2D.ALLOW_IMAGE_READ);
		offScreenCanvas3D.setOffScreenBuffer(imageComponent);

		return offScreenCanvas3D;
	}

	public JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();

		fileMenu = new JMenu("R3");
		fileMenu.add(createMenuItem("R3", "Randomize", this));
		fileMenu.add(showAxis);
		fileMenu.add(createMenuItem("R3", "Exit", this));
		menuBar.add(fileMenu);

		JMenu sliderMenu = new JMenu("Sensitivity");
		slider = new RubikSliderMenuElement();
		slider.addChangeListener(this);
		sliderMenu.add(slider);
		menuBar.add(sliderMenu);

		return menuBar;
	}

	private JMenuItem createMenuItem(String menuText, String buttonText,
			ActionListener listener) {
		JMenuItem menuItem = new JMenuItem(buttonText);
		menuItem.addActionListener(listener);
		menuItem.setActionCommand(menuText + "|" + buttonText);
		return menuItem;
	}

	protected BranchGroup createViewBranchGroup(TransformGroup[] tgArray,
			ViewPlatform vp) {
		BranchGroup vpBranchGroup = new BranchGroup();

		if (tgArray != null && tgArray.length > 0) {
			Group parentGroup = vpBranchGroup;
			TransformGroup curTg = null;

			for (int n = 0; n < tgArray.length; n++) {
				curTg = tgArray[n];
				parentGroup.addChild(curTg);
				parentGroup = curTg;
			}

			tgArray[tgArray.length - 1].addChild(vp);
		} else
			vpBranchGroup.addChild(vp);

		return vpBranchGroup;
	}

	protected ViewPlatform createViewPlatform() {
		ViewPlatform vp = new ViewPlatform();
		vp.setViewAttachPolicy(View.RELATIVE_TO_FIELD_OF_VIEW);
		vp.setActivationRadius(100);

		return vp;
	}

	public TransformGroup[] getViewTransformGroupArray() {
		tgArray[1] = new TransformGroup();

		// move the camera BACK a little...
		// note that we have to invert the matrix as
		// we are moving the viewer
		Transform3D t3d = new Transform3D();
		t3d.setScale(2);
		t3d.setTranslation(new Vector3d(0.0, 0.0, -1.6));
		t3d.invert();
		tgArray[1].setTransform(t3d);

		tgArray[0] = new TransformGroup();
		tgArray[0].setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		tgArray[0].setTransform(t3d_axis_false);

		return tgArray;
	}

	static protected void registerWindowListener(JFrame frame) {
		// disable automatic close support for Swing frame.
		frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		// adds the window listener
		frame.addWindowListener(new WindowAdapter() {
			// handles the system exit window message
			public void windowClosing(WindowEvent e) {
				System.exit(1);
			}
		});
	}

	public static void main(String[] args) {
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);

		ToolTipManager ttm = ToolTipManager.sharedInstance();
		ttm.setLightWeightPopupEnabled(false);

		JFrame frame = new JFrame();

		RubikApplication rubikApplet = new RubikApplication();
		frame.setJMenuBar(rubikApplet.createMenuBar());

		frame.getContentPane().add(rubikApplet);
		frame.setSize(550, 550);
		registerWindowListener(frame);

		frame.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		System.out.println("Action Performed: " + ae.getActionCommand());

		java.util.StringTokenizer toker = new java.util.StringTokenizer(ae
				.getActionCommand(), "|");

		String menu = toker.nextToken();
		String command = toker.nextToken();

		if (menu.equals("R3")) {
			if (command.equals("Exit")) {
				System.exit(0);
			}
			else if (command.equals("Randomize")) {
				cube3D.randomize();
			}
			else if (command.equals("Show Axis")) {
				fileMenu.remove(showAxis);
				fileMenu.add(hideAxis, 1);
				cube3D.setAxis(true);
				tgArray[0].setTransform(t3d_axis_true);
			}
			else if (command.equals("Hide Axis")) {
				fileMenu.remove(hideAxis);
				fileMenu.add(showAxis, 1);
				cube3D.setAxis(false);
				tgArray[0].setTransform(t3d_axis_false);
			}
		}
	}

	@Override
	public void stateChanged(ChangeEvent e) {
		if(!slider.getValueIsAdjusting()) {
			cube3D.setMouseSensitivity((double) (0.02d + 2*slider.getValue()/1000));
		}
	}

}
