/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.ui;

import java.util.Map;

import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.IndexedLineArray;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TransparencyAttributes;
import javax.vecmath.Color3f;
import javax.vecmath.Color4f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import rubik.core.RubikDirection;

import com.sun.j3d.utils.behaviors.mouse.MouseRotate;

public class RubikAxis {

	BranchGroup axisRoot = null;
	BranchGroup axisBG = null;
	TransformGroup axisTG = null;
	TransformGroup mouseRotateTG = null;
	
	BranchGroup sideBG = null;
	Map<RubikDirection, Color3f> dirtoColor = null;

	public RubikAxis(BranchGroup root, Map<RubikDirection, Color3f> dirtoColorMap) {

		axisRoot = new BranchGroup();
		axisRoot.setCapability(BranchGroup.ALLOW_DETACH);

		Shape3D axisLines = new Shape3D();
		axisLines.setGeometry(createAxisLines());

		axisBG = new BranchGroup();
		axisBG.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		axisBG.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		axisBG.addChild(axisLines);

		mouseRotateTG = new TransformGroup();
		mouseRotateTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		mouseRotateTG.addChild(axisBG);
		mouseRotateTG.setTransform(new Transform3D());

		axisTG = new TransformGroup();
		axisTG.addChild(mouseRotateTG);
		Transform3D trans = new Transform3D();
		trans.setScale(0.3);
		trans.setTranslation(new Vector3d(-1.0, 0, 0));
		axisTG.setTransform(trans);
		
		dirtoColor = dirtoColorMap;
		
		sideBG = new BranchGroup();
		sideBG.setCapability(BranchGroup.ALLOW_DETACH);
		createSideDisplay();

		axisRoot.addChild(axisTG);
		root.addChild(axisRoot);
	}
	
	public void updateAxisRotateTG(Transform3D rotateTransform) {
		mouseRotateTG.setTransform(rotateTransform);
	}
	
	public void refresh(Map<RubikDirection, Color3f> map) {
		dirtoColor = map;
		
		axisBG.removeChild(sideBG);
		
		sideBG = new BranchGroup();
		sideBG.setCapability(BranchGroup.ALLOW_DETACH);
		
		createSideDisplay();
	}
	private void createSideDisplay() {
		
		
		TransformGroup temp_TG = new TransformGroup();
		Transform3D temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(0.75f, 0f, 0f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideX(dirtoColor.get(RubikDirection.X_P)));
		sideBG.addChild(temp_TG);
		
		temp_TG = new TransformGroup();
		temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(-0.75f, 0f, 0f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideX(dirtoColor.get(RubikDirection.X_N)));
		sideBG.addChild(temp_TG);
		
		temp_TG = new TransformGroup();
		temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(0f, -0.75f, 0f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideY(dirtoColor.get(RubikDirection.Y_N)));
		sideBG.addChild(temp_TG);
		
		temp_TG = new TransformGroup();
		temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(0f, 0.75f, 0f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideY(dirtoColor.get(RubikDirection.Y_P)));
		sideBG.addChild(temp_TG);
		
		temp_TG = new TransformGroup();
		temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(0f, 0f, -0.75f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideZ(dirtoColor.get(RubikDirection.Z_N)));
		sideBG.addChild(temp_TG);
		
		temp_TG = new TransformGroup();
		temp_trans = new Transform3D();
		temp_trans.setTranslation(new Vector3f(0f, 0f, 0.75f));
		temp_TG.setTransform(temp_trans);
		temp_TG.addChild(createSideZ(dirtoColor.get(RubikDirection.Z_P)));
		sideBG.addChild(temp_TG);
		
		axisBG.addChild(sideBG);
	}

	private Shape3D createSideX(Color3f color) {

		Shape3D root = new Shape3D();
		Color4f color4 = new Color4f(color.getX(), color.getY(), color.getZ(), 0.7f);
		
		QuadArray rect1;

		rect1 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect1.setCoordinate(0, new Point3d(0, -0.2, -0.2));
		rect1.setCoordinate(1, new Point3d(0, -0.2, 0.2));
		rect1.setCoordinate(2, new Point3d(0, 0.2, 0.2));
		rect1.setCoordinate(3, new Point3d(0, 0.2, -0.2));

		rect1.setColor(0, color4);
		rect1.setColor(1, color4);
		rect1.setColor(2, color4);
		rect1.setColor(3, color4);
		
		QuadArray rect2;
		
		rect2 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect2.setCoordinate(0, new Point3d(0, -0.2, -0.2));
		rect2.setCoordinate(1, new Point3d(0, 0.2, -0.2));
		rect2.setCoordinate(2, new Point3d(0, 0.2, 0.2));
		rect2.setCoordinate(3, new Point3d(0, -0.2, 0.2));

		rect2.setColor(0, color4);
		rect2.setColor(1, color4);
		rect2.setColor(2, color4);
		rect2.setColor(3, color4);
		
		root.addGeometry(rect2);
		root.addGeometry(rect1);
		
		TransparencyAttributes transparency = new TransparencyAttributes(
				TransparencyAttributes.BLENDED, 0.3f, TransparencyAttributes.BLEND_SRC_ALPHA,
				TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
		
		Appearance appearance = new Appearance();
		appearance.setTransparencyAttributes(transparency);
		
		root.setAppearance(appearance);

		return root;

	}
	
	private Shape3D createSideY(Color3f color) {

		Shape3D root = new Shape3D();
		Color4f color4 = new Color4f(color.getX(), color.getY(), color.getZ(), 0.7f);
		
		QuadArray rect1;

		rect1 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect1.setCoordinate(0, new Point3d(-0.2, 0, -0.2));
		rect1.setCoordinate(1, new Point3d(-0.2, 0, 0.2));
		rect1.setCoordinate(2, new Point3d(0.2, 0, 0.2));
		rect1.setCoordinate(3, new Point3d(0.2, 0, -0.2));

		rect1.setColor(0, color4);
		rect1.setColor(1, color4);
		rect1.setColor(2, color4);
		rect1.setColor(3, color4);
		
		QuadArray rect2;
		
		rect2 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect2.setCoordinate(0, new Point3d(-0.2, 0, -0.2));
		rect2.setCoordinate(1, new Point3d(0.2, 0, -0.2));
		rect2.setCoordinate(2, new Point3d(0.2, 0, 0.2));
		rect2.setCoordinate(3, new Point3d(-0.2, 0, 0.2));

		rect2.setColor(0, color4);
		rect2.setColor(1, color4);
		rect2.setColor(2, color4);
		rect2.setColor(3, color4);
		
		root.addGeometry(rect2);
		root.addGeometry(rect1);
		
		TransparencyAttributes transparency = new TransparencyAttributes(
				TransparencyAttributes.BLENDED, 0.3f, TransparencyAttributes.BLEND_SRC_ALPHA,
				TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
		
		Appearance appearance = new Appearance();
		appearance.setTransparencyAttributes(transparency);
		
		root.setAppearance(appearance);

		return root;

	}
	
	private Shape3D createSideZ(Color3f color) {

		Shape3D root = new Shape3D();
		
		Color4f color4 = new Color4f(color.getX(), color.getY(), color.getZ(), 0.7f);
		
		QuadArray rect1;

		rect1 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect1.setCoordinate(0, new Point3d(-0.2, -0.2, 0));
		rect1.setCoordinate(1, new Point3d(0.2, -0.2, 0));
		rect1.setCoordinate(2, new Point3d(0.2, 0.2, 0));
		rect1.setCoordinate(3, new Point3d(-0.2, 0.2, 0));

		rect1.setColor(0, color4);
		rect1.setColor(1, color4);
		rect1.setColor(2, color4);
		rect1.setColor(3, color4);
		
		QuadArray rect2;
		
		rect2 = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_4);
		rect2.setCoordinate(0, new Point3d(-0.2, -0.2, 0));
		rect2.setCoordinate(1, new Point3d(-0.2, 0.2, 0));
		rect2.setCoordinate(2, new Point3d(0.2, 0.2, 0));
		rect2.setCoordinate(3, new Point3d(0.2, -0.2, 0));

		rect2.setColor(0, color4);
		rect2.setColor(1, color4);
		rect2.setColor(2, color4);
		rect2.setColor(3, color4);
		
		root.addGeometry(rect2);
		root.addGeometry(rect1);
		
		TransparencyAttributes transparency = new TransparencyAttributes(
				TransparencyAttributes.BLENDED, 0.3f, TransparencyAttributes.BLEND_SRC_ALPHA,
				TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
		
		Appearance appearance = new Appearance();
		appearance.setTransparencyAttributes(transparency);
		
		root.setAppearance(appearance);

		return root;

	}

	private Geometry createAxisLines() {
		IndexedLineArray axisLines = new IndexedLineArray(18,
				GeometryArray.COORDINATES, 30);

		axisLines.setCoordinate( 0, new Point3f(-1.0f, 0.0f, 0.0f));
		axisLines.setCoordinate( 1, new Point3f( 1.0f, 0.0f, 0.0f));
		axisLines.setCoordinate( 2, new Point3f( 0.9f, 0.1f, 0.1f));
		axisLines.setCoordinate( 3, new Point3f( 0.9f,-0.1f, 0.1f));
		axisLines.setCoordinate( 4, new Point3f( 0.9f, 0.1f,-0.1f));
		axisLines.setCoordinate( 5, new Point3f( 0.9f,-0.1f,-0.1f));
		axisLines.setCoordinate( 6, new Point3f( 0.0f,-1.0f, 0.0f));
		axisLines.setCoordinate( 7, new Point3f( 0.0f, 1.0f, 0.0f));
		axisLines.setCoordinate( 8, new Point3f( 0.1f, 0.9f, 0.1f));
		axisLines.setCoordinate( 9, new Point3f(-0.1f, 0.9f, 0.1f));
		axisLines.setCoordinate(10, new Point3f( 0.1f, 0.9f,-0.1f));
		axisLines.setCoordinate(11, new Point3f(-0.1f, 0.9f,-0.1f));
		axisLines.setCoordinate(12, new Point3f( 0.0f, 0.0f,-1.0f));
		axisLines.setCoordinate(13, new Point3f( 0.0f, 0.0f, 1.0f));
		axisLines.setCoordinate(14, new Point3f( 0.1f, 0.1f, 0.9f));
		axisLines.setCoordinate(15, new Point3f(-0.1f, 0.1f, 0.9f));
		axisLines.setCoordinate(16, new Point3f( 0.1f,-0.1f, 0.9f));
		axisLines.setCoordinate(17, new Point3f(-0.1f,-0.1f, 0.9f));

		axisLines.setCoordinateIndex( 0, 0);
		axisLines.setCoordinateIndex( 1, 1);
		axisLines.setCoordinateIndex( 2, 2);
		axisLines.setCoordinateIndex( 3, 1);
		axisLines.setCoordinateIndex( 4, 3);
		axisLines.setCoordinateIndex( 5, 1);
		axisLines.setCoordinateIndex( 6, 4);
		axisLines.setCoordinateIndex( 7, 1);
		axisLines.setCoordinateIndex( 8, 5);
		axisLines.setCoordinateIndex( 9, 1);
		axisLines.setCoordinateIndex(10, 6);
		axisLines.setCoordinateIndex(11, 7);
		axisLines.setCoordinateIndex(12, 8);
		axisLines.setCoordinateIndex(13, 7);
		axisLines.setCoordinateIndex(14, 9);
		axisLines.setCoordinateIndex(15, 7);
		axisLines.setCoordinateIndex(16,10);
		axisLines.setCoordinateIndex(17, 7);
		axisLines.setCoordinateIndex(18,11);
		axisLines.setCoordinateIndex(19, 7);
		axisLines.setCoordinateIndex(20,12);
		axisLines.setCoordinateIndex(21,13);
		axisLines.setCoordinateIndex(22,14);
		axisLines.setCoordinateIndex(23,13);
		axisLines.setCoordinateIndex(24,15);
		axisLines.setCoordinateIndex(25,13);
		axisLines.setCoordinateIndex(26,16);
		axisLines.setCoordinateIndex(27,13);
		axisLines.setCoordinateIndex(28,17);
		axisLines.setCoordinateIndex(29,13);

		return axisLines;

	}

}
