/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.ui;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Group;
import javax.media.j3d.Material;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import rubik.core.RubikBlock;
import rubik.core.RubikColor;
import rubik.core.RubikDirection;

public class RubikBlock3D {

	BranchGroup blockRoot = null;
	TransformGroup blockAlignmentTG = null;
	TransformGroup blockRotationTG = null;
	RubikBlock block = null;

	static Material mat_blue = null;
	static Material mat_white = null;
	static Material mat_orange = null;
	static Material mat_yellow = null;
	static Material mat_green = null;
	static Material mat_red = null;

	public RubikBlock3D(RubikBlock Rblock) {
		block = Rblock;

		if(mat_white == null) {
			mat_white = new Material();
			mat_white.setShininess(70f);
			mat_white.setLightingEnable(true);
			mat_white.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_white.setEmissiveColor(getColor3f(RubikColor.WHITE));
		}

		if(mat_blue == null) {
			mat_blue = new Material();
			mat_blue.setShininess(70f);
			mat_blue.setLightingEnable(true);
			mat_blue.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_blue.setEmissiveColor(getColor3f(RubikColor.BLUE));
		}

		if(mat_green == null) {
			mat_green = new Material();
			mat_green.setShininess(70f);
			mat_green.setLightingEnable(true);
			mat_green.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_green.setEmissiveColor(getColor3f(RubikColor.GREEN));
		}

		if(mat_red == null) {
			mat_red = new Material();
			mat_red.setShininess(70f);
			mat_red.setLightingEnable(true);
			mat_red.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_red.setEmissiveColor(getColor3f(RubikColor.RED));
		}

		if(mat_yellow == null) {
			mat_yellow = new Material();
			mat_yellow.setShininess(70f);
			mat_yellow.setLightingEnable(true);
			mat_yellow.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_yellow.setEmissiveColor(getColor3f(RubikColor.YELLOW));
		}

		if(mat_orange == null) {
			mat_orange = new Material();
			mat_orange.setShininess(70f);
			mat_orange.setLightingEnable(true);
			mat_orange.setColorTarget(Material.AMBIENT_AND_DIFFUSE);
			mat_orange.setEmissiveColor(getColor3f(RubikColor.ORANGE));
		}

	}

	Material getMat(RubikColor col) {
		if(col == RubikColor.BLUE) {
			return mat_blue;
		}
		else if(col == RubikColor.ORANGE) {
			return mat_orange;
		}
		else if(col == RubikColor.RED) {
			return mat_red;
		}
		else if(col == RubikColor.WHITE) {
			return mat_white;
		}
		else if(col == RubikColor.GREEN) {
			return mat_green;
		}
		else {
			return mat_yellow;
		}
	}

	public BranchGroup createBlock() {

		blockRoot = new BranchGroup();

		blockRoot.setCapability(BranchGroup.ALLOW_DETACH);
		blockRoot.setCapability(Group.ALLOW_CHILDREN_READ);

		blockRotationTG = new TransformGroup();
		blockRotationTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		blockAlignmentTG = new TransformGroup();
		blockAlignmentTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

		Shape3D translated1 = new Shape3D();
		Shape3D translated2 = new Shape3D();
		Shape3D translated3 = new Shape3D();
		Shape3D translated4 = new Shape3D();
		Shape3D translated5 = new Shape3D();
		Shape3D translated6 = new Shape3D();


		Color3f temp = null;

		temp = getColor3f(block.getSide(RubikDirection.X_N).getColor());
		translated4.addGeometry(createRectXN(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.X_N).getColor()));
			translated4.setAppearance(appearance);
		}

		temp = getColor3f(block.getSide(RubikDirection.Y_N).getColor());
		translated5.addGeometry(createRectYN(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.Y_N).getColor()));
			translated5.setAppearance(appearance);
		}

		temp = getColor3f(block.getSide(RubikDirection.Z_N).getColor());
		translated6.addGeometry(createRectZN(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.Z_N).getColor()));
			translated6.setAppearance(appearance);
		}

		temp = getColor3f(block.getSide(RubikDirection.X_P).getColor());
		translated1.addGeometry(createRectXP(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.X_P).getColor()));
			translated1.setAppearance(appearance);
		}

		temp = getColor3f(block.getSide(RubikDirection.Y_P).getColor());
		translated2.addGeometry(createRectYP(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.Y_P).getColor()));
			translated2.setAppearance(appearance);
		}

		temp = getColor3f(block.getSide(RubikDirection.Z_P).getColor());
		translated3.addGeometry(createRectZP(temp));
		if(!(temp.getX() == 0.5 && temp.getY() == 0.5 && temp.getZ() == 0.5)) {

			Appearance appearance = new Appearance();
			appearance.setMaterial(getMat(block.getSide(RubikDirection.Z_P).getColor()));
			translated3.setAppearance(appearance);
		}

		blockRotationTG.addChild(translated1);
		blockRotationTG.addChild(translated2);
		blockRotationTG.addChild(translated3);
		blockRotationTG.addChild(translated4);
		blockRotationTG.addChild(translated5);
		blockRotationTG.addChild(translated6);

		refreshAlignment();

		blockAlignmentTG.addChild(blockRotationTG);
		blockRoot.addChild(blockAlignmentTG);

		return blockRoot;
	}

	private QuadArray createRectXN(Color3f color) {

		QuadArray curRect;

		Vector3f rectNormal = new Vector3f(-1.0f, 0, 0);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(-0.095, -0.095, -0.095));
		curRect.setCoordinate(1, new Point3d(-0.095, -0.095, 0.095));
		curRect.setCoordinate(2, new Point3d(-0.095, 0.095, 0.095));
		curRect.setCoordinate(3, new Point3d(-0.095, 0.095, -0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	private QuadArray createRectXP(Color3f color) {

		QuadArray curRect;

		Vector3f rectNormal = new Vector3f(1.0f, 0, 0);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(0.095, -0.095, -0.095));
		curRect.setCoordinate(1, new Point3d(0.095, 0.095, -0.095));
		curRect.setCoordinate(2, new Point3d(0.095, 0.095, 0.095));
		curRect.setCoordinate(3, new Point3d(0.095, -0.095, 0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	private QuadArray createRectYN(Color3f color) {

		QuadArray curRect;

		Vector3f rectNormal = new Vector3f(0, -1.0f, 0);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(-0.095, -0.095, -0.095));
		curRect.setCoordinate(1, new Point3d(0.095, -0.095, -0.095));
		curRect.setCoordinate(2, new Point3d(0.095, -0.095, 0.095));
		curRect.setCoordinate(3, new Point3d(-0.095, -0.095, 0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	private QuadArray createRectYP(Color3f color) {

		QuadArray curRect;

		Vector3f rectNormal = new Vector3f(0, 1.0f, 0);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(-0.095, 0.095, -0.095));
		curRect.setCoordinate(1, new Point3d(-0.095, 0.095, 0.095));
		curRect.setCoordinate(2, new Point3d(0.095, 0.095, 0.095));
		curRect.setCoordinate(3, new Point3d(0.095, 0.095, -0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	private QuadArray createRectZN(Color3f color) {

		QuadArray curRect;
		Vector3f rectNormal = new Vector3f(0, 0, -1.0f);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(-0.095, -0.095, -0.095));
		curRect.setCoordinate(1, new Point3d(-0.095, 0.095, -0.095));
		curRect.setCoordinate(2, new Point3d(0.095, 0.095, -0.095));
		curRect.setCoordinate(3, new Point3d(0.095, -0.095, -0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	private QuadArray createRectZP(Color3f color) {

		QuadArray curRect;

		Vector3f rectNormal = new Vector3f(0, 0, 1.0f);

		curRect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.NORMALS | GeometryArray.COLOR_3);
		curRect.setCoordinate(0, new Point3d(-0.095, -0.095, 0.095));
		curRect.setCoordinate(1, new Point3d(0.095, -0.095, 0.095));
		curRect.setCoordinate(2, new Point3d(0.095, 0.095, 0.095));
		curRect.setCoordinate(3, new Point3d(-0.095, 0.095, 0.095));

		curRect.setColor(0, color);
		curRect.setColor(1, color);
		curRect.setColor(2, color);
		curRect.setColor(3, color);

		curRect.setNormal(0, rectNormal);
		curRect.setNormal(1, rectNormal);
		curRect.setNormal(2, rectNormal);
		curRect.setNormal(3, rectNormal);

		return curRect;
	}

	public RubikBlock getBlock() {
		return block;
	}

	public BranchGroup getBlockRoot() {
		return blockRoot;
	}

	public Color3f getColor3f(RubikColor Rcolor) {

		Color3f col = new Color3f();

		if(Rcolor == RubikColor.BLUE) {
			col.set(0, 0, 0.7f);
		}
		else if(Rcolor == RubikColor.GREEN) {
			col.set(0, 0.7f, 0);
		}
		else if(Rcolor == RubikColor.RED) {
			col.set(0.7f, 0, 0);
		}
		else if(Rcolor == RubikColor.ORANGE) {
			col.set(0.7f, 0.3f, 0);
		}
		else if(Rcolor == RubikColor.WHITE) {
			col.set(0.7f, 0.7f, 0.7f);
		}
		else if(Rcolor == RubikColor.YELLOW) {
			col.set(0.7f, 0.7f, 0);
		}
		else
		{
			col.set(0.5f, 0.5f, 0.5f);
		}
		return col;
	}

	private void refreshAlignment() {

		Transform3D temp_align = new Transform3D();
		temp_align.set(new Vector3d(block.getCoordinates().getx() * 0.2 - 0.2,
				block.getCoordinates().gety() * 0.2 - 0.2, 
				block.getCoordinates().getz() * 0.2 - 0.2));
		blockAlignmentTG.setTransform(temp_align);

	}


	public void setCoordinates(Integer[] COORDS) {
		block.setCoordinates(COORDS);

		refreshAlignment();
	}

	public void setRotation(int x, int y, int z) {

		Transform3D temp_transform1 = new Transform3D();
		Transform3D temp_transform2 = new Transform3D();

		blockRotationTG.getTransform(temp_transform1);

		if(x == 1) {
			temp_transform2.rotX(Math.PI/2);
		}
		else if(x == -1) {
			temp_transform2.rotX(-1*Math.PI/2);
		}
		else if(y == 1) {
			temp_transform2.rotY(Math.PI/2);
		}
		else if(y == -1) {
			temp_transform2.rotY(-1*Math.PI/2);
		}
		else if(z == 1) {
			temp_transform2.rotZ(Math.PI/2);
		}
		else if(z == -1) {
			temp_transform2.rotZ(-1*Math.PI/2);
		}

		block.setRotation(x, y, z);
		temp_transform2.mul(temp_transform1);
		blockRotationTG.setTransform(temp_transform2);

	}
}
