/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.ui;

import java.awt.Font;
import java.awt.event.KeyEvent;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Font3D;
import javax.media.j3d.FontExtrusion;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.QuadArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Text3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TransparencyAttributes;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

public class RubikControlsUIArrow {
	
	double Beta = Math.PI/4;
	double cos_Beta = Math.cos(Beta);
	double sin_Beta = Math.sin(Beta);
	
	TransformGroup initialRotation = null;
	TransformGroup arrowTranslateTG = null;
	TransformGroup arrowRotateTG = null;
	BranchGroup text_BG = null;
	TransformGroup text_3TG = null;
	TransformGroup text_5TG = null;
	
	BranchGroup arrowBG = null;
	boolean isAttached = false;
	Appearance appearance = null;
	Shape3D arrowShapeRoot = null;
	Color3f arrowColor = null;
	int attachKeyCode = -1;
	Vector3d normal = null;
	Vector3d origPos = null;
	Vector3d circleCenter = null;
	
	Transform3D currentMouseTransform = null;
	
	BranchGroup sceneroot = null;
	BranchGroup root = null;
	
	public RubikControlsUIArrow(BranchGroup root, BranchGroup sceneroot) {
		
		origPos = new Vector3d();
		normal = new Vector3d();
		circleCenter = new Vector3d();
		
		arrowBG = new BranchGroup();
		arrowBG.setCapability(BranchGroup.ALLOW_DETACH);
	
		arrowRotateTG = new TransformGroup();
		arrowRotateTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		arrowBG.addChild(arrowRotateTG);
		
		arrowTranslateTG = new TransformGroup();
		arrowTranslateTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		arrowRotateTG.addChild(arrowTranslateTG);
		
		initialRotation = new TransformGroup();
		initialRotation.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		
		arrowTranslateTG.addChild(initialRotation);
		
		text_BG = new BranchGroup();
		text_BG.setCapability(BranchGroup.ALLOW_DETACH);
		
		text_3TG = new TransformGroup();
		text_3TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		text_BG.addChild(text_3TG);
		
		text_5TG = new TransformGroup();
		text_5TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		text_BG.addChild(text_5TG);
				
		this.sceneroot = sceneroot;
		this.root = root;
		
		TransparencyAttributes transparency = new TransparencyAttributes(
				TransparencyAttributes.BLENDED, 0.4f, TransparencyAttributes.BLEND_SRC_ALPHA,
				TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
		
		appearance = new Appearance();
		appearance.setTransparencyAttributes(transparency);
		
		arrowShapeRoot = new Shape3D();
		arrowShapeRoot.setAppearance(appearance);
		
		arrowColor = new Color3f(0.5f, 0.5f, 0.5f);
		
		createArrow();
		
		initialRotation.addChild(arrowShapeRoot);
	}
	
	void createArrow() {
		
		int numSections = 12;
		
		double y, z, x;
		double theta, theta1;
		
		Point3d p1 = new Point3d();
		Point3d p2 = new Point3d();
		Point3d p3 = new Point3d();
		Point3d p4 = new Point3d();
		
        Font3D font = new Font3D(new Font("Helvetica", Font.PLAIN, 1),
                new FontExtrusion());
        
        Text3D textGeom = null;
        Shape3D textShape = null;
		
		for(int i=0; i<numSections; i++) {
			
			theta = Math.PI/(2*numSections)*i - Math.PI/4;
			theta1 = Math.PI/(2*numSections)*(i+1) - Math.PI/4;
			
			y = 0.45 * Math.sin(theta);
			z = 0.45 * Math.cos(theta) - 0.45;
			if(i < (int)numSections/6) {
				x = 0.05 * i*6/numSections;
			}
			else if(i >= (int)5*numSections/6) {
				x = 0.05 - 0.05 * (i-numSections+numSections/6)*6/numSections;
			}
			else {
				x = 0.03;
			}
			
			p1.set(-1*x, y, z);
			p2.set(x, y, z);
			
			y = 0.45 * Math.sin(theta1);
			z = 0.45 * Math.cos(theta1) - 0.45;
			if(i < (int)numSections/6) {
				x = 0.05 * (i+1)*6/numSections;
			}
			else if(i >= (int)5*numSections/6) {
				x = 0.05 - 0.05 * (i+1-numSections+numSections/6)*6/numSections;
			}
			else {
				x = 0.03;
			}
			p3.set(x, y, z);
			p4.set(-1*x, y, z);
			
			createTransparentWhiteRectangle(p1, p2, p3, p4);
			
			createTransparentWhiteRectangle(p1, p4, p3, p2);
			
		}
		
        textGeom = new Text3D(font, new String("3"));
        textGeom.setAlignment(Text3D.ALIGN_CENTER);
        textShape = new Shape3D();
        textShape.setGeometry(textGeom);
        text_3TG.addChild(textShape);
        
        textGeom = new Text3D(font, new String("5"));
        textGeom.setAlignment(Text3D.ALIGN_CENTER);
        textShape = new Shape3D();
        textShape.setGeometry(textGeom);
        text_5TG.addChild(textShape);
		
	}

	public void updateArrow(Transform3D transform) {
		if(!transform.equals(currentMouseTransform)) {
			currentMouseTransform = transform;
		}
		
		if(isAttached) {

			Transform3D temp = null;
			Transform3D rotationTransform = new Transform3D();
			Vector3d transformedNormal = new Vector3d();
			Vector3d transformedArrowCenter = new Vector3d();
			Vector3d transformedCircleCenter = new Vector3d();
			
			// C is the center of the circle of radius 0.45 and 
			// in the plane of the selected side
			double Cx, Cy, Cz;
			// u is the vector pointing from C to point P on the 
			// circle with max. z component 
			double ux, uy, uz;
			// v is the tangent to the circle at point P (max. z component on circle)
			double vx, vy, vz;
			// V is the current vector from C to the current arrow center
			double Vx, Vy, Vz;
			// w is the cross product of u and V
			double wx, wy, wz;
			// t is the vector perpendicular to the plane of circle
			double tx, ty, tz;
			// theta is the angle between u and V
			double theta;
			// used  for normalizing vectors
			double r;

			transformedNormal.set(normal.getX(), normal.getY(), normal.getZ());
			transform.transform(transformedNormal);

			tx = transformedNormal.getX();
			ty = transformedNormal.getY();
			tz = transformedNormal.getZ();

			transformedArrowCenter.set(origPos.getX(), origPos.getY(), origPos.getZ());
			transform.transform(transformedArrowCenter);

			transformedCircleCenter.set(circleCenter.getX(), circleCenter.getY(), circleCenter.getZ());
			transform.transform(transformedCircleCenter);

			Cx = transformedCircleCenter.getX();
			Cy = transformedCircleCenter.getY();
			Cz = transformedCircleCenter.getZ();

			// get the transformed normal to circle

			tx = tx - Cx;
			ty = ty - Cy;
			tz = tz - Cz;


			if(tx == 0 && ty == 0) { // this condition normally occurs when e,d or c
				// rotations are used without rotating the cube using mouse
				// in all other cases, the else block is executed
				
				// get the vector pointing from circle center to min. x point on the circle

				uy = -1 * ty*tx;
				ux = tz*tz + ty*ty;
				uz = -1 * tx*tz;
				
				// calculate the scaled vector tangent to the circle at point with min. x component

				r = Math.sqrt(tz * tz + ty * ty);
				vy = -0.45 * tz / r ;
				vz = 0.45 * ty / r;
				vx = 0;

			}
			else {

				// get the vector pointing from circle center to max. z point on the circle

				uz = -1* (tx*tx + ty*ty);
				uy = tz*ty;
				ux = tz*tx;

				// calculate the scaled vector tangent to the circle at point with max. z component 

				r = Math.sqrt(tx * tx + ty * ty);
				vx = 0.45 * ty / r ;
				vy = -0.45 * tx / r;
				vz = 0;
			}

			// scale u to the GUI scaling

			r = Math.sqrt(ux*ux+uy*uy+uz*uz);
			ux = 0.45 * ux/r;
			uy = 0.45 * uy/r;
			uz = 0.45 * uz/r;

			// get the vector pointing from circle center to actual arrow position

			Vx = transformedArrowCenter.getX() - Cx;
			Vy = transformedArrowCenter.getY() - Cy;
			Vz = transformedArrowCenter.getZ() - Cz;

			// calculate the angle between the actual and required arrow position (angle between V and u)

			theta = Math.acos(-1*(ux*Vx + uy*Vy + uz*Vz)/(0.45 * Math.sqrt(Vx*Vx+Vy*Vy+Vz*Vz)));

			// check if theta is NaN, if yes set it to 0

			if(Double.isNaN(theta)) {
				theta = 0;
			}

			// cross product between u and V

			wx = uy*Vz - Vy*uz;
			wy = uz*Vx - ux*Vz;
			wz = ux*Vy - Vx*uy;

			// t . (V x u) -- this will help resolve theta from 
			//0-2pi (arccos resolves theta only in the range 0-pi)

			if((wx*tx + wy*ty + wz*tz) < 0) {
				theta = 2*Math.PI - theta;
			}

			// calculate positions of text "3" and "5" using u, v and C

			temp = new Transform3D();
			temp.set(new Vector3d(Cx - sin_Beta * ux + cos_Beta * vx, Cy - sin_Beta * uy + cos_Beta * vy, Cz - sin_Beta * uz + cos_Beta * vz));
			temp.setScale(0.06d);
			text_3TG.setTransform(temp);

			temp = new Transform3D();
			temp.set(new Vector3d(Cx - sin_Beta * vx - cos_Beta * ux, Cy - sin_Beta * vy - cos_Beta * uy, Cz - cos_Beta * uz - sin_Beta * vz));
			temp.setScale(0.06d);
			text_5TG.setTransform(temp);


			switch(attachKeyCode) {
			case KeyEvent.VK_E:
				rotationTransform.rotZ(theta);
				break;
			case KeyEvent.VK_D: 
				rotationTransform.rotZ(theta);
				break;
			case KeyEvent.VK_C: 
				rotationTransform.rotZ(theta);
				break;
			case KeyEvent.VK_R: 
				rotationTransform.rotX(theta);
				break;
			case KeyEvent.VK_F: 
				rotationTransform.rotX(theta);
				break;
			case KeyEvent.VK_V: 
				rotationTransform.rotX(theta);
				break;
			case KeyEvent.VK_T: 
				rotationTransform.rotY(theta);
				break;
			case KeyEvent.VK_G: 
				rotationTransform.rotY(theta);
				break;
			case KeyEvent.VK_B: 
				rotationTransform.rotY(theta);
				break;
			}
			
			// rotate the arrow by theta on the circle
			
			arrowRotateTG.setTransform(rotationTransform);
		}
	}
	
	public void attach(int eventKeyCode) {

		if(attachKeyCode != eventKeyCode) {
			attachKeyCode = eventKeyCode;
			
			if(isAttached) {
				detach();
				Transform3D temp = new Transform3D();
				arrowRotateTG.setTransform(temp);
			}
			
			Transform3D arrowRotation = new Transform3D();
			Transform3D arrowTranslation = new Transform3D();

			switch(eventKeyCode) {
			case KeyEvent.VK_E:
				origPos.set(-0.45, 0, 0.2);
				arrowRotation.rotY(-1*Math.PI/2);
				normal.set(0, 0, 1);
				circleCenter.set(0, 0, 0.2);
				break;
			case KeyEvent.VK_D: 
				origPos.set(-0.45, 0, 0);
				arrowRotation.rotY(-1*Math.PI/2);
				normal.set(0, 0, 1);
				circleCenter.set(0, 0, 0);
				break;
			case KeyEvent.VK_C: 
				origPos.set(-0.45, 0, -0.2);
				arrowRotation.rotY(-1*Math.PI/2);
				normal.set(0, 0, 1);
				circleCenter.set(0, 0, -0.2);
				break;
			case KeyEvent.VK_R: 
				origPos.set(0.2, 0, 0.45);
				normal.set(1, 0, 0);
				circleCenter.set(0.2, 0, 0);
				break;
			case KeyEvent.VK_F: 
				origPos.set(0, 0, 0.45);
				normal.set(1, 0, 0);
				circleCenter.set(0, 0, 0);
				break;
			case KeyEvent.VK_V: 
				origPos.set(-0.2, 0, 0.45);
				normal.set(1, 0, 0);
				circleCenter.set(-0.2, 0, 0);
				break;
			case KeyEvent.VK_T: 
				origPos.set(0, 0.2, 0.45);
				arrowRotation.rotZ(Math.PI/2);
				normal.set(0, 1, 0);
				circleCenter.set(0, 0.2, 0);
				break;
			case KeyEvent.VK_G: 
				origPos.set(0, 0, 0.45);
				arrowRotation.rotZ(Math.PI/2);
				normal.set(0, 1, 0);
				circleCenter.set(0, 0, 0);
				break;
			case KeyEvent.VK_B: 
				origPos.set(0, -0.2, 0.45);
				arrowRotation.rotZ(Math.PI/2);
				normal.set(0, 1, 0);
				circleCenter.set(0, -0.2, 0);
				break;
			}

			arrowTranslation.set(origPos);
			initialRotation.setTransform(arrowRotation);
			arrowTranslateTG.setTransform(arrowTranslation);

			sceneroot.addChild(arrowBG);
			root.addChild(text_BG);

			isAttached = true;
			if(currentMouseTransform == null) {
				updateArrow(new Transform3D());
			}
			else {
				updateArrow(currentMouseTransform);
			}
		}
		
	}
	
	void createTransparentWhiteRectangle(Point3d p1, Point3d p2, Point3d p3, Point3d p4) {
		
		QuadArray rect = new QuadArray(4, GeometryArray.COORDINATES | GeometryArray.COLOR_3);
		
		rect.setCoordinate(0, new Point3d(p1.getX(), p1.getY(), p1.getZ()));
		rect.setCoordinate(1, new Point3d(p2.getX(), p2.getY(), p2.getZ()));
		rect.setCoordinate(2, new Point3d(p3.getX(), p3.getY(), p3.getZ()));
		rect.setCoordinate(3, new Point3d(p4.getX(), p4.getY(), p4.getZ()));
		
		rect.setColor(0, arrowColor);
		rect.setColor(1, arrowColor);
		rect.setColor(2, arrowColor);
		rect.setColor(3, arrowColor);
		
		arrowShapeRoot.addGeometry(rect);
		
	}
	
	public void detach() {
		
		attachKeyCode = 0;
		isAttached = false;
		sceneroot.removeChild(arrowBG);
		root.removeChild(text_BG);
	}

}
