/*******************************************************************************
 *    Rubik's Cube 3D game
 *     Copyright (C) 2011  Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com)
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 * 
 * 
 * Contributors:
 * Harshad Sahasrabudhe (harshad.sahasrabudhe@gmail.com) - Core and UI development
 * Vishwajit Gaikwad (vishrockon7@gmail.com) - Testing
 ******************************************************************************/
package rubik.ui;

import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.media.j3d.Behavior;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Group;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOnElapsedFrames;
import javax.swing.Timer;
import javax.vecmath.Color3f;

import rubik.core.RubikBlock;
import rubik.core.RubikBlockID;
import rubik.core.RubikCube;
import rubik.core.RubikDirection;
import rubik.core.RubikRotationMaps;
import rubik.core.RubikRotationList;

public class RubikCube3D {

	public class KeyTriggerBehavior extends Behavior {

		private WakeupCriterion AWTEventCondition;
		private WakeupCriterion Wakeupnextframe;
		private boolean rotationSelected = false;

		private BranchGroup rotationBG = null;
		TransformGroup rotationTG = null;
		public Transform3D rotationMatrix = null;
		int rotationKeyCode, directionKeyCode;
		int numFrames = 0;
		int framesPerRotation = 30;
		Timer timer = null;
		boolean update = false;
		boolean reundo = false;

		boolean robotWorks = true;
		boolean processingKeyEvent = false;

		RubikAnimationCallBack callBackClass = null;

		ActionListener setUpdate = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				update = true;
			}
		};

		public void setCallBack(RubikAnimationCallBack c) {
			this.callBackClass =  c;
		}

		public KeyTriggerBehavior() {
			AWTEventCondition = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);
			Wakeupnextframe = new WakeupOnElapsedFrames(0);
		}

		private void end() {
			Map<Integer[], Integer> newCoordinateMap = new HashMap<Integer[], Integer>();
			List<Integer[]> coordstobeRemoved = new LinkedList<Integer[]>();
			Integer[] tempCoords = null;
			Integer[] newCoords = null;
			Integer tempVal = null;
			if(rotationKeyCode == KeyEvent.VK_E ||
					rotationKeyCode == KeyEvent.VK_D ||
					rotationKeyCode == KeyEvent.VK_C) {
				for(RubikBlock block: Rcube.blocks) {
					if( (block.getCoordinates().getz() == 2 && rotationKeyCode == KeyEvent.VK_E) ||
							(block.getCoordinates().getz() == 1 && rotationKeyCode == KeyEvent.VK_D) ||
							(block.getCoordinates().getz() == 0 && rotationKeyCode == KeyEvent.VK_C) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = Rcube.coordinateMap.get(tempCoords);

						rotationTG.removeChild(allBlocks3D.get(tempVal).getBlockRoot());

						if(directionKeyCode == KeyEvent.VK_3) {
							newCoords = RubikRotationMaps.getZPMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(0, 0, 1);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getZNMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(0, 0, -1);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}

						sceneroot.addChild(allBlocks3D.get(tempVal).getBlockRoot());
					}
				}

				if(rotationKeyCode == KeyEvent.VK_D && enableAxis) {
					axis.refresh(getSideCenterColors());
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_R ||
					rotationKeyCode == KeyEvent.VK_F ||
					rotationKeyCode == KeyEvent.VK_V) {
				for(RubikBlock block: Rcube.blocks) {
					if( (block.getCoordinates().getx() == 2 && rotationKeyCode == KeyEvent.VK_R) ||
							(block.getCoordinates().getx() == 1 && rotationKeyCode == KeyEvent.VK_F) ||
							(block.getCoordinates().getx() == 0 && rotationKeyCode == KeyEvent.VK_V) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = Rcube.coordinateMap.get(tempCoords);

						rotationTG.removeChild(allBlocks3D.get(tempVal).getBlockRoot());

						if(directionKeyCode == KeyEvent.VK_3) {
							newCoords = RubikRotationMaps.getXPMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(1, 0, 0);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getXNMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(-1, 0, 0);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}

						sceneroot.addChild(allBlocks3D.get(tempVal).getBlockRoot());
					}
				}
				if(rotationKeyCode == KeyEvent.VK_F && enableAxis) {
					axis.refresh(getSideCenterColors());
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_T ||
					rotationKeyCode == KeyEvent.VK_G ||
					rotationKeyCode == KeyEvent.VK_B)  {
				for(RubikBlock block: Rcube.blocks) {
					if( (block.getCoordinates().gety() == 2 && rotationKeyCode == KeyEvent.VK_T) ||
							(block.getCoordinates().gety() == 1 && rotationKeyCode == KeyEvent.VK_G) ||
							(block.getCoordinates().gety() == 0 && rotationKeyCode == KeyEvent.VK_B) ) {

						tempCoords = block.getCoordinates_Iarray();
						tempVal = Rcube.coordinateMap.get(tempCoords);

						rotationTG.removeChild(allBlocks3D.get(tempVal).getBlockRoot());

						if(directionKeyCode == KeyEvent.VK_3) {
							newCoords = RubikRotationMaps.getYPMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(0, 1, 0);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}
						else {
							newCoords = RubikRotationMaps.getYNMap().get(tempCoords);

							allBlocks3D.get(tempVal).setRotation(0, -1, 0);
							allBlocks3D.get(tempVal).setCoordinates(newCoords);

							newCoordinateMap.put(newCoords, tempVal);
							coordstobeRemoved.add(tempCoords);
						}

						sceneroot.addChild(allBlocks3D.get(tempVal).getBlockRoot());
					}
				}
				if(rotationKeyCode == KeyEvent.VK_G && enableAxis) {
					axis.refresh(getSideCenterColors());
				}
			}

			sceneroot.removeChild(rotationBG);

			for(Integer[] coords: coordstobeRemoved) {
				Rcube.coordinateMap.remove(coords);
			}
			Rcube.coordinateMap.putAll(newCoordinateMap);

			System.gc();

		}

		@Override
		public void initialize() {
			this.wakeupOn(AWTEventCondition);

		}

		void createTimer() {
			update = false;
			if(timer == null) {
				timer = new Timer(15, setUpdate);
				timer.setRepeats(false);
				timer.start();
			}
			else {
				timer.restart();
			}
		}

		public void processKeyEvent(int keyCode, boolean reundo, int nextKeyCode) {

			if(keyCode == KeyEvent.VK_E ||
					keyCode == KeyEvent.VK_D ||
					keyCode == KeyEvent.VK_C ||
					keyCode == KeyEvent.VK_R ||
					keyCode == KeyEvent.VK_F ||
					keyCode == KeyEvent.VK_V ||
					keyCode == KeyEvent.VK_T ||
					keyCode == KeyEvent.VK_G ||
					keyCode == KeyEvent.VK_B) {

				rotationSelected = true;
				rotationKeyCode = keyCode;
				rubikControlsArrow.detach();
				rubikControlsArrow.attach(keyCode);

				if(reundo) {
					processKeyEvent(nextKeyCode, false, 0);
				}
				else {
					this.wakeupOn(AWTEventCondition);
				}
			}
			else if(rotationSelected && (keyCode == KeyEvent.VK_3 ||
					keyCode == KeyEvent.VK_5 )) {

				redoCommands.getList().clear();

				directionKeyCode = keyCode;

				rotationMatrix = new Transform3D();

				rotationTG = new TransformGroup();
				rotationTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
				rotationTG.setCapability(Group.ALLOW_CHILDREN_EXTEND);
				rotationTG.setCapability(Group.ALLOW_CHILDREN_WRITE);
				rotationTG.setTransform(new Transform3D());

				numFrames = 0;

				putCubesinRotateTG();

				rotationBG = new BranchGroup();
				rotationBG.setCapability(BranchGroup.ALLOW_DETACH);
				rotationBG.addChild(rotationTG);
				sceneroot.addChild(rotationBG);

				createTimer();
				processingKeyEvent = true;
				this.wakeupOn(Wakeupnextframe);
			}
			else if(keyCode == KeyEvent.VK_U) {
				if(!undoCommands.isEmpty()) {
					this.reundo = true;
					int command = undoCommands.pop();
					if(command == RubikRotationList.e3) {
						processKeyEvent(KeyEvent.VK_E, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.e5);
					}
					else if(command == RubikRotationList.e5){
						processKeyEvent(KeyEvent.VK_E, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.e3);
					}
					else if(command == RubikRotationList.d3){
						processKeyEvent(KeyEvent.VK_D, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.d5);
					}
					else if(command == RubikRotationList.d5){
						processKeyEvent(KeyEvent.VK_D, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.d3);
					}
					else if(command == RubikRotationList.c3){
						processKeyEvent(KeyEvent.VK_C, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.c5);
					}
					else if(command == RubikRotationList.c5){
						processKeyEvent(KeyEvent.VK_C, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.c3);
					}
					else if(command == RubikRotationList.r3){
						processKeyEvent(KeyEvent.VK_R, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.r5);
					}
					else if(command == RubikRotationList.r5){
						processKeyEvent(KeyEvent.VK_R, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.r3);
					}
					else if(command == RubikRotationList.f3){
						processKeyEvent(KeyEvent.VK_F, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.f5);
					}
					else if(command == RubikRotationList.f5){
						processKeyEvent(KeyEvent.VK_F, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.f3);
					}
					else if(command == RubikRotationList.v3){
						processKeyEvent(KeyEvent.VK_V, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.v5);
					}
					else if(command == RubikRotationList.v5){
						processKeyEvent(KeyEvent.VK_V, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.v3);
					}
					else if(command == RubikRotationList.t3){
						processKeyEvent(KeyEvent.VK_T, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.t5);
					}
					else if(command == RubikRotationList.t5){
						processKeyEvent(KeyEvent.VK_T, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.t3);
					}
					else if(command == RubikRotationList.g3){
						processKeyEvent(KeyEvent.VK_G, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.g5);
					}
					else if(command == RubikRotationList.g5){
						processKeyEvent(KeyEvent.VK_G, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.g3);
					}
					else if(command == RubikRotationList.b3){
						processKeyEvent(KeyEvent.VK_B, true, KeyEvent.VK_3);
						redoCommands.add(RubikRotationList.b5);
					}
					else if(command == RubikRotationList.b5){
						processKeyEvent(KeyEvent.VK_B, true, KeyEvent.VK_5);
						redoCommands.add(RubikRotationList.b3);
					}
					userCommands.pop();
				}
				else {
					this.wakeupOn(AWTEventCondition);
				}
			}
			else if(keyCode == KeyEvent.VK_I) {
				if(!redoCommands.isEmpty()) {
					this.reundo = true;
					int command = redoCommands.pop();
					if(command == RubikRotationList.e3) {
						processKeyEvent(KeyEvent.VK_E, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.e3);
						undoCommands.add(RubikRotationList.e5);
					}
					else if(command == RubikRotationList.e5){
						processKeyEvent(KeyEvent.VK_E, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.e5);
						undoCommands.add(RubikRotationList.e3);
					}
					else if(command == RubikRotationList.d3){
						processKeyEvent(KeyEvent.VK_D, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.d3);
						undoCommands.add(RubikRotationList.d5);
					}
					else if(command == RubikRotationList.d5){
						processKeyEvent(KeyEvent.VK_D, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.d5);
						undoCommands.add(RubikRotationList.d3);
					}
					else if(command == RubikRotationList.c3){
						processKeyEvent(KeyEvent.VK_C, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.c3);
						undoCommands.add(RubikRotationList.c5);
					}
					else if(command == RubikRotationList.c5){
						processKeyEvent(KeyEvent.VK_C, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.c5);
						undoCommands.add(RubikRotationList.c3);
					}
					else if(command == RubikRotationList.r3){
						processKeyEvent(KeyEvent.VK_R, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.r3);
						undoCommands.add(RubikRotationList.r5);
					}
					else if(command == RubikRotationList.r5){
						processKeyEvent(KeyEvent.VK_R, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.r5);
						undoCommands.add(RubikRotationList.r3);
					}
					else if(command == RubikRotationList.f3){
						processKeyEvent(KeyEvent.VK_F, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.f3);
						undoCommands.add(RubikRotationList.f5);
					}
					else if(command == RubikRotationList.f5){
						processKeyEvent(KeyEvent.VK_F, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.f5);
						undoCommands.add(RubikRotationList.f3);
					}
					else if(command == RubikRotationList.v3){
						processKeyEvent(KeyEvent.VK_V, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.v3);
						undoCommands.add(RubikRotationList.v5);
					}
					else if(command == RubikRotationList.v5){
						processKeyEvent(KeyEvent.VK_V, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.v5);
						undoCommands.add(RubikRotationList.v3);
					}
					else if(command == RubikRotationList.t3){
						processKeyEvent(KeyEvent.VK_T, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.t3);
						undoCommands.add(RubikRotationList.t5);
					}
					else if(command == RubikRotationList.t5){
						processKeyEvent(KeyEvent.VK_T, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.t5);
						undoCommands.add(RubikRotationList.t3);
					}
					else if(command == RubikRotationList.g3){
						processKeyEvent(KeyEvent.VK_G, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.g3);
						undoCommands.add(RubikRotationList.g5);
					}
					else if(command == RubikRotationList.g5){
						processKeyEvent(KeyEvent.VK_G, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.g5);
						undoCommands.add(RubikRotationList.g3);
					}
					else if(command == RubikRotationList.b3){
						processKeyEvent(KeyEvent.VK_B, true, KeyEvent.VK_3);
						userCommands.add(RubikRotationList.b3);
						undoCommands.add(RubikRotationList.b5);
					}
					else if(command == RubikRotationList.b5){
						processKeyEvent(KeyEvent.VK_B, true, KeyEvent.VK_5);
						userCommands.add(RubikRotationList.b5);
						undoCommands.add(RubikRotationList.b3);
					}
				}
				else {
					this.wakeupOn(AWTEventCondition);
				}
			}
			else {
				this.wakeupOn(AWTEventCondition);
			}
		}

		@Override
		public void processStimulus(Enumeration criteria) {
			WakeupCriterion criterion = (WakeupCriterion) criteria.nextElement();
			if(criterion.equals(AWTEventCondition)) {
				AWTEvent[] event = ((WakeupOnAWTEvent)criterion).getAWTEvent();


				for(int i=0; i<event.length; i++) {
					if(event[i] instanceof KeyEvent) {
						if(!processingKeyEvent) {
							processKeyEvent(((KeyEvent)event[i]).getKeyCode(), false, 0);
						}
					}
				}
			}
			else if(criterion.equals(Wakeupnextframe)) {

				if(numFrames < framesPerRotation) {
					if(update) {
						Transform3D tempTransform = new Transform3D();
						rotationTG.getTransform(tempTransform);
						tempTransform.mul(rotationMatrix);
						rotationTG.setTransform(tempTransform);
						numFrames++;
						createTimer();
					}
					this.wakeupOn(Wakeupnextframe);
				}
				else {
					end();
					processingKeyEvent = false;
					this.wakeupOn(AWTEventCondition);
					if(callBackClass != null) {
						callBackClass.callBackFunction();
					}
				}

			}


		}

		private void putCubesinRotateTG() {


			if(rotationKeyCode == KeyEvent.VK_E) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getz() == 2) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotZ(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.e3);
						undoCommands.add(RubikRotationList.e5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotZ(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.e5);
						undoCommands.add(RubikRotationList.e3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_D) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getz() == 1) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotZ(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.d3);
						undoCommands.add(RubikRotationList.d5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotZ(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.d5);
						undoCommands.add(RubikRotationList.d3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_C) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getz() == 0) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotZ(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.c3);
						undoCommands.add(RubikRotationList.c5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotZ(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.c5);
						undoCommands.add(RubikRotationList.c3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_R) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getx() == 2) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotX(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.r3);
						undoCommands.add(RubikRotationList.r5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotX(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.r5);
						undoCommands.add(RubikRotationList.r3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_F) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getx() == 1) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotX(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.f3);
						undoCommands.add(RubikRotationList.f5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotX(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.f5);
						undoCommands.add(RubikRotationList.f3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_V) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().getx() == 0) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotX(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.v3);
						undoCommands.add(RubikRotationList.v5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotX(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.v5);
						undoCommands.add(RubikRotationList.v3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_T) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().gety() == 2) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotY(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.t3);
						undoCommands.add(RubikRotationList.t5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotY(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.t5);
						undoCommands.add(RubikRotationList.t3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_G) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().gety() == 1) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotY(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.g3);
						undoCommands.add(RubikRotationList.g5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotY(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.g5);
						undoCommands.add(RubikRotationList.g3);
					}
					else {
						reundo = false;
					}
				}
			}
			else if(rotationKeyCode == KeyEvent.VK_B) {
				for(RubikBlock block: Rcube.blocks) {
					if(block.getCoordinates().gety() == 0) {
						sceneroot.removeChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
						rotationTG.addChild(allBlocks3D.get(
								Rcube.coordinateMap.get(block.getCoordinates_Iarray())).getBlockRoot());
					}
				}
				if(directionKeyCode == KeyEvent.VK_3) {
					rotationMatrix.rotY(Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.b3);
						undoCommands.add(RubikRotationList.b5);
					}
					else {
						reundo = false;
					}
				}
				else {
					rotationMatrix.rotY(-1*Math.PI/60);
					if(!reundo) {
						userCommands.add(RubikRotationList.b5);
						undoCommands.add(RubikRotationList.b3);
					}
					else {
						reundo = false;
					}
				}
			}
		}	
	}

	private Map<RubikDirection, Color3f> getSideCenterColors() {
		Map<RubikDirection, Color3f> dirtoColor = new HashMap<RubikDirection, Color3f>();

		for(RubikBlock3D block: allBlocks3D) {
			if(block.getBlock().getID() == RubikBlockID.SIDE_CENTER) {
				if(block.getBlock().getCoordinates().getx() == 2) {
					dirtoColor.put(RubikDirection.X_P, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.X_P).getColor()));
				}
				else if(block.getBlock().getCoordinates().getx() == 0) {
					dirtoColor.put(RubikDirection.X_N, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.X_N).getColor()));
				}
				else if(block.getBlock().getCoordinates().gety() == 2) {
					dirtoColor.put(RubikDirection.Y_P, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.Y_P).getColor()));
				}
				else if(block.getBlock().getCoordinates().gety() == 0) {
					dirtoColor.put(RubikDirection.Y_N, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.Y_N).getColor()));
				}
				else if(block.getBlock().getCoordinates().getz() == 2) {
					dirtoColor.put(RubikDirection.Z_P, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.Z_P).getColor()));
				}
				else if(block.getBlock().getCoordinates().getz() == 0) {
					dirtoColor.put(RubikDirection.Z_N, 
							block.getColor3f(block.getBlock().getSide(RubikDirection.Z_N).getColor()));
				}
			}
		}

		return dirtoColor;
	}

	public void setAxis(boolean enable) {
		if(enable) {
			if(axis == null) {
				axis = new RubikAxis(root, getSideCenterColors());
				Transform3D currentRotation = new Transform3D();
				mouseRotate.mouseRotateTG.getTransform(currentRotation);
				axis.updateAxisRotateTG(currentRotation);
				rubikControls.setAxisObject(axis);
			}
			else {
				Transform3D currentRotation = new Transform3D();
				axis.refresh(getSideCenterColors());
				//mouseRotate.mouseRotateTG.getTransform(currentRotation);
				//axis.updateAxisRotateTG(currentRotation);
				root.addChild(axis.axisRoot);
			}
			enableAxis = true;
		}
		else {
			if(axis != null) {
				root.removeChild(axis.axisRoot);
			}
			enableAxis = false;
		}
	}

	RubikRotationList userCommands = null;
	RubikRotationList undoCommands = null;
	RubikRotationList redoCommands = null;

	private RubikCube Rcube = null;
	private BranchGroup sceneroot = null;
	private List<RubikBlock3D> allBlocks3D = null;
	private KeyTriggerBehavior trigger = null;

	public BoundingSphere bounds = null;

	RubikAxis axis = null;

	RubikMouseRotate mouseRotate = null;

	RubikControlsUI rubikControls = null;

	RubikControlsUIArrow rubikControlsArrow = null;

	boolean enableAxis = true;

	BranchGroup root;

	public RubikCube3D(BranchGroup root) {

		this.root = root;

		bounds = new BoundingSphere();

		sceneroot = new BranchGroup();
		sceneroot.setCapability(Group.ALLOW_CHILDREN_EXTEND);
		sceneroot.setCapability(Group.ALLOW_CHILDREN_WRITE);

		mouseRotate = new RubikMouseRotate(root, sceneroot);

		rubikControls = new RubikControlsUI(root);

		mouseRotate.setCallBack(
				rubikControls.getCallbackClass());

		rubikControlsArrow = new RubikControlsUIArrow(root, sceneroot);

		rubikControls.setArrowObject(rubikControlsArrow);

		Rcube = new RubikCube();
		Rcube.randomize(20);
		allBlocks3D = new LinkedList<RubikBlock3D>();
		RubikBlock3D blockCreator = null;
		for(RubikBlock curBlock: Rcube.blocks) {
			blockCreator = new RubikBlock3D(curBlock);
			blockCreator.createBlock();
			sceneroot.addChild(blockCreator.getBlockRoot());
			allBlocks3D.add(blockCreator);
		}

		trigger = new KeyTriggerBehavior();
		trigger.setSchedulingBounds(bounds);
		sceneroot.addChild(trigger);

		//		if(enableAxis) {
		//			axis = new RubikAxis(root, getSideCenterColors());
		//		}

		userCommands = new RubikRotationList();
		undoCommands = new RubikRotationList();
		redoCommands = new RubikRotationList();

	}

	public void randomize() {

		for(RubikBlock3D blockBranchGroup: allBlocks3D) {
			sceneroot.removeChild(blockBranchGroup.blockRoot);
		}
		allBlocks3D.clear();

		Rcube = new RubikCube();
		Rcube.randomize(20);
		allBlocks3D = new LinkedList<RubikBlock3D>();
		RubikBlock3D blockCreator = null;
		for(RubikBlock curBlock: Rcube.blocks) {
			blockCreator = new RubikBlock3D(curBlock);
			blockCreator.createBlock();
			sceneroot.addChild(blockCreator.getBlockRoot());
			allBlocks3D.add(blockCreator);
		}

		userCommands.getList().clear();
		undoCommands.getList().clear();
		redoCommands.getList().clear();

		if(enableAxis) {
			axis.refresh(getSideCenterColors());
		}
	}
	
	public void setMouseSensitivity(double sen) {
		mouseRotate.myMouseRotate.setFactor(sen);
	}


}
